package com.mh.eems.advice;

import java.io.Serializable;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.entity.SysLogEntity;
import com.mh.eems.service.SysLogService;

import io.swagger.annotations.ApiOperation;

/**
 * 统一日志处理
 * 
 * @author 小威老师 xiaoweijiagou@163.com
 *
 *         2017年8月19日
 */
@Aspect
@Component
public class LogAdvice {

	@Autowired
	private SysLogService logService;

	@Around(value = "@annotation(com.mh.eems.annotation.LogAnnotation)")
	public Object logSave(ProceedingJoinPoint joinPoint) throws Throwable {
		SysLogEntity sysLogs = new SysLogEntity();
		MethodSignature methodSignature = (MethodSignature) joinPoint.getSignature();

		String module = null;
		String logs = null;
		String type = null;
		LogAnnotation logAnnotation = methodSignature.getMethod().getDeclaredAnnotation(LogAnnotation.class);
		module = logAnnotation.module();
		logs = logAnnotation.logs();
		type = logAnnotation.type();
		if (StringUtils.isEmpty(module)) {
			ApiOperation apiOperation = methodSignature.getMethod().getDeclaredAnnotation(ApiOperation.class);
			if (apiOperation != null) {
				module = apiOperation.value();
			}
		}

		if (StringUtils.isEmpty(module)) {
			throw new RuntimeException("没有指定日志module");
		}
		sysLogs.setModule(module);
		sysLogs.setLogs(logs);
		sysLogs.setType(type);

		try {
			Object object = joinPoint.proceed();

			sysLogs.setGrade(1);
			logService.insertLogs(sysLogs);

			return object;
		} catch (Exception e) {
			sysLogs.setGrade(0);
			sysLogs.setRemark(e.getMessage());
			logService.insertLogs(sysLogs);
			throw e;
		}

	}
}
