package com.mh.eems.controller;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletRequest;

import com.mh.eems.util.DateUtil;
import org.apache.ibatis.annotations.Select;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.metadata.Sheet;
import com.alibaba.fastjson.JSON;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.entity.DeviceInstallEntity;
import com.mh.eems.entity.Excel2DeviceInstallEntity;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.UserEntity;
import com.mh.eems.service.DeviceInstallService;
import com.mh.eems.util.JsonPackUtils;
import com.mh.eems.util.StringUtil;

/**
 * @ClassName: DeviceInstallController
 * @Description:TODO(基表安装访问接口)
 * @author: 铭汉科技—LJF
 * @date: 2019年8月7日 上午10:20:07
 */

@RestController
@RequestMapping("/deviceInstall")
public class DeviceInstallController {

    @Autowired
    private DeviceInstallService deviceInstallService;

    // 通过excel批量导入数据
    @RequestMapping("/importExp")
    @ResponseBody
    public <T> Object importExp(@RequestParam("file") MultipartFile file, HttpServletRequest request) {
        // 判断文件是否为空
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
        if (!file.isEmpty()) {
            try {
                String originalFilename = file.getOriginalFilename();//原文件名字
                System.out.println("文件名：" + originalFilename);
                InputStream inputStream = file.getInputStream();//获取输入流
                BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
                DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
                List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2DeviceInstallEntity.class));        // 第几张工作表，第几行开始，javamodel
                inputStream.close();
                // 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
                int resultInt = 0;
                int batchCount = 150;// 每批commit的个数
                int batchLastIndex = batchCount;// 每批最后一个的下标
                for (int index = 0; index < data.size(); ) {
                    if (batchLastIndex >= data.size()) {
                        batchLastIndex = data.size();
                        resultInt = resultInt * deviceInstallService.deviceBatchInsert(data.subList(index, batchLastIndex));
                        System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
                        break;// 数据插入完毕，退出循环
                    } else {
                        resultInt = resultInt * deviceInstallService.deviceBatchInsert(data.subList(index, batchLastIndex));
                        System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
                        index = batchLastIndex;// 设置下一批下标
                        batchLastIndex = index + (batchCount - 1);
                    }
                }
                System.out.println("批量插入长度：" + data.size());
//		        int a = userService.batchInsert(data);
                if (batchLastIndex == data.size()) {
                    object = JSON.toJSON(jsonPackUtils.ok("导入" + originalFilename + "成功"));
                }
                StringUtil stringUtil = new StringUtil();
                stringUtil.print(data);
            } catch (Exception e) {
                object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
                e.printStackTrace();
            }
        }
        return object;
    }

    // 通过excel批量导入数据
    @RequestMapping("/importExpTest")
    @ResponseBody
    public <T> Object importExpTest(@RequestParam("file") MultipartFile file, HttpServletRequest request) {
        // 判断文件是否为空
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
        if (!file.isEmpty()) {
            try {
                String originalFilename = file.getOriginalFilename();//原文件名字
                System.out.println("文件名  ：" + originalFilename);
                System.out.println("文件名  ：" + originalFilename);
                InputStream inputStream = file.getInputStream();//获取输入流
                BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
                DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
                List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2DeviceInstallEntity.class));        // 第几张工作表，第几行开始，javamodel
                inputStream.close();
                object = JSON.toJSON(jsonPackUtils.ok("导入excel成功"));
                StringUtil stringUtil = new StringUtil();
                stringUtil.print(data);
            } catch (Exception e) {
                object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
                e.printStackTrace();
            }
        }
        return object;
    }

    // // 满足根据户号查询出对应的楼栋名称、楼栋ID值、表身号和抄表标识码
    @GetMapping("/queryByHouseNum")
    public <T> Object queryByHouseNum(@RequestParam(value = "houseNum", required = false) String houseNum) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        List<UserEntity> list = deviceInstallService.queryDeviceByHouseNum(houseNum);
        ResponseEntity<UserEntity> responseEntity = new ResponseEntity<UserEntity>();
        responseEntity.setCode(200);
        responseEntity.setMsg("success");
        responseEntity.setData(list);
        return JSON.toJSON(responseEntity);

    }

    // 查询全部的基表安装信息
    @LogAnnotation(module = "基表管理", logs = "查询全部基表信息内容", type = "查询")
    @GetMapping("/queryAll")
    @Cacheable("queryAllDevices")
    public <T> Object queryAllDevices(@RequestParam(value = "page", required = false) Integer page,
                                      @RequestParam(value = "limit", required = false) Integer limit) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        List<DeviceInstallEntity> list = deviceInstallService.queryDevices(page, limit);
        ResponseEntity<DeviceInstallEntity> responseEntity = new ResponseEntity<DeviceInstallEntity>();
        responseEntity.setCode(200);
        responseEntity.setMsg("success");
        responseEntity.setData(list);
        DeviceInstallEntity deviceInstallEntity = new DeviceInstallEntity();
        deviceInstallEntity.setLastValue(1.0);
        responseEntity.setCount(deviceInstallService.queryCount(deviceInstallEntity, null, null));
        return JSON.toJSON(responseEntity);
    }

    // 添加基表信息
    @PostMapping("/add")
    @ResponseBody
    public <T> Object addDevice(@RequestBody DeviceInstallEntity deviceInstallEntity) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        try {
            String a = deviceInstallService.insertInstall(deviceInstallEntity);
            if (a.equals("success")) {
                object = JSON.toJSON(jsonPackUtils.ok("添加" + deviceInstallEntity.getHouseNum() + "成功"));
            } else {
                object = JSON.toJSON(jsonPackUtils.error("添加失败" + a));
            }

        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("添加" + deviceInstallEntity.getHouseNum() + "失败"));
            e.printStackTrace();
        }
        return object;
    }

    // 修改基表信息
    @PostMapping("/update")
    @ResponseBody
    public <T> Object updateDevice(@RequestBody DeviceInstallEntity deviceInstallEntity) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        try {
            int a = deviceInstallService.updateInstalls(deviceInstallEntity);
            if (a != 0) {
                object = JSON.toJSON(jsonPackUtils.ok("更新" + deviceInstallEntity.getHouseNum() + "成功"));
            } else {
                object = JSON.toJSON(jsonPackUtils.error("更新" + deviceInstallEntity.getHouseNum() + "失败"));
            }

        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("更新" + deviceInstallEntity.getHouseNum() + "失败"));
            e.printStackTrace();
        }
        return object;
    }

    // 删除基表信息
    @RequestMapping("/delete")
    public <T> Object deleteDevice(@RequestParam(value = "houseNum", required = false) String houseNum) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        try {
            int a = deviceInstallService.deleteInstall(houseNum);
            if (a != 0) {
                object = JSON.toJSON(jsonPackUtils.ok("删除成功"));
            } else {
                object = JSON.toJSON(jsonPackUtils.error("删除失败"));
            }
        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("删除失败"));
            e.printStackTrace();
        }
        return object;
    }

    // 根据条件查询对应的基表信息
    @LogAnnotation(module = "基表管理", logs = "根据条件查询符合内容的基表信息内容", type = "查询")
    @RequestMapping("/queryByType")        // update by ljf on 2019-09-29 导出查询结果集的全部基表信息，传page、limit为空

    public <T> Object queryByType(
            @RequestParam(value = "page", required = false) Integer page,
            @RequestParam(value = "limit", required = false) Integer limit,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "houseNum", required = false) String houseNum,
            @RequestParam(value = "buildingName", required = false) String buildingName,
            @RequestParam(value = "deviceNum", required = false) String deviceNum,
            @RequestParam(value = "paramName", required = false) String paramName,
            @RequestParam(value = "communicationType", required = false) String communicationType,
            @RequestParam(value = "dataCom", required = false) String dataCom,
            @RequestParam(value = "deviceCaliber", required = false) String deviceCaliber,
            @RequestParam(value = "last_value", required = false) Double last_value,
            @RequestParam(value = "lastDate", required = false) String lastDate,
            @RequestParam(value = "status", required = false) String status) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        DeviceInstallEntity deviceInstallEntity = new DeviceInstallEntity();
        if (houseNum != null && houseNum != "") {
            deviceInstallEntity.setHouseNum(houseNum);
        }
        if (buildingName != null && buildingName != "") {
            deviceInstallEntity.setBuildingName(buildingName);
        }
        if (deviceNum != null && deviceNum != "") {
            deviceInstallEntity.setDeviceNum(deviceNum);
        }
        if (paramName != null && paramName != "") {
            deviceInstallEntity.setParamName(paramName);
        }
        if (communicationType != null && communicationType != "") {
            deviceInstallEntity.setCommunicationType(communicationType);
        }
        if (deviceCaliber != null && deviceCaliber != "") {
            deviceInstallEntity.setDeviceCaliber(deviceCaliber);
        }
        if (dataCom != null && dataCom != "") {
            deviceInstallEntity.setDataCom(dataCom);
        }
        if (lastDate != null && lastDate != "") {
            System.out.println(lastDate);
            deviceInstallEntity.setLastDate(DateUtil.strToDateLong(lastDate));
        }
        if (last_value == null) {
            deviceInstallEntity.setLastValue(1.0);
        } else {
            deviceInstallEntity.setLastValue(last_value);
        }
        if (status != null && status != "") {
            deviceInstallEntity.setStatus(status);
        }

        List<DeviceInstallEntity> list = deviceInstallService.queryType(deviceInstallEntity, page, limit, startTime, endTime);
        ResponseEntity<DeviceInstallEntity> responseEntity = new ResponseEntity<DeviceInstallEntity>();
        responseEntity.setCode(200);
        responseEntity.setMsg("success");
        responseEntity.setData(list);
        responseEntity.setCount(deviceInstallService.queryCount(deviceInstallEntity, startTime, endTime));
        System.out.println(JSON.toJSON(responseEntity));
        return JSON.toJSON(responseEntity);
    }
}
