package com.mh.eems.controller;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.alibaba.fastjson.JSON;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.entity.DeviceParamEntity;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.service.DeviceParamService;
import com.mh.eems.util.JsonPackUtils;

/**
 * 
 * @ClassName:  DeviceParamController   
 * @Description:TODO(页面接口层)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年7月31日 上午8:42:12   
 *
 */
@RestController
@CrossOrigin(origins = "*",maxAge = 3600)
@RequestMapping("/deviceParam")
public class DeviceParamController {
	
	@Autowired
	private DeviceParamService deviceParamService;
	
	// 查询全部用户
	@LogAnnotation(module="设备参数", logs = "查询全部设备参数", type = "查询")

	@GetMapping("/queryAll")
	public Object queryAllDeviceParams(@RequestParam(value="page", required=false)Integer page,
			@RequestParam(value="limit", required=false)Integer limit) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		DeviceParamEntity deviceParamEntity = new DeviceParamEntity();
		ResponseEntity<DeviceParamEntity> responseEntity = new ResponseEntity<DeviceParamEntity>();
		List<DeviceParamEntity> deviceParamEntities = deviceParamService.queryDeviceParamsByOthers(deviceParamEntity, page, limit);
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(deviceParamEntities);
		responseEntity.setCount(deviceParamService.queryDeviceParamsCount(deviceParamEntity, page, limit));
		System.out.println( JSON.toJSON(responseEntity).toString());
		return JSON.toJSON(responseEntity);
	}
	
	// 根据输入条件查询
	@LogAnnotation(module="设备参数", logs = "按条件查询对应设备参数", type = "查询")
	@GetMapping("/queryByOther")
	public Object queryDeviceParamsByOther(@RequestParam(value="paramId", required=false)Integer paramId,
			@RequestParam(value="paramName", required=false)String paramName,
			@RequestParam(value="paramBaudrate", required=false)Integer paramBaudrate,
			@RequestParam(value="page", required=false)Integer page,
			@RequestParam(value="limit", required=false)Integer limit) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		DeviceParamEntity deviceParamEntity = new DeviceParamEntity();
		if (paramId != null) {
			deviceParamEntity.setParamId((long)paramId);
		}
		if (paramName != null) {
			deviceParamEntity.setParamName(paramName);
		}
		if (paramBaudrate != null) {
			deviceParamEntity.setParamBaudrate((long)paramBaudrate);
		}
		ResponseEntity<DeviceParamEntity> responseEntity = new ResponseEntity<DeviceParamEntity>();
		List<DeviceParamEntity> deviceParamEntities = deviceParamService.queryDeviceParamsByOthers(deviceParamEntity, page, limit);
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(deviceParamEntities);
		responseEntity.setCount(deviceParamService.queryDeviceParamsCount(deviceParamEntity, page, limit));
		return JSON.toJSON(responseEntity);
	}
	
	// 增加参数信息 
	@GetMapping("/add")
	public <T> Object addDeviceParam(@RequestParam(value="paramId", required=false)Integer paramId,
			@RequestParam(value="paramName", required=false)String paramName,
			@RequestParam(value="paramBaudrate", required=false)Integer paramBaudrate) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		DeviceParamEntity deviceParamEntity = new DeviceParamEntity();
		deviceParamEntity.setParamId((long)paramId);
		deviceParamEntity.setParamName(paramName);
		deviceParamEntity.setParamBaudrate((long)paramBaudrate);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		DeviceParamEntity DeviceParamEntity = null;
		try {
			DeviceParamEntity = deviceParamService.queryDeviceParamsById(deviceParamEntity);
			if (DeviceParamEntity != null) {
				object = JSON.toJSON(jsonPackUtils.error("已存在当前ID值"+deviceParamEntity.getParamId())); 
				System.out.println(JSON.toJSON(object));
			} else {
					deviceParamService.insertDeviceParam(deviceParamEntity);
					object = JSON.toJSON(jsonPackUtils.ok("添加"+deviceParamEntity.getParamName()+"成功")); 
					System.out.println(JSON.toJSON(object));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("添加"+deviceParamEntity.getParamName()+"错误")); 
			e.printStackTrace();
		}
		return object;
	}
	
	// 修改参数信息
	@RequestMapping("/update")
	public <T> Object updateDeviceParam(@RequestParam(value = "paramId", required = false) Integer paramId,
			@RequestParam(value = "paramName", required = false) String paramName,
			@RequestParam(value = "paramBaudrate", required = false) Integer paramBaudrate) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		DeviceParamEntity deviceParamEntity = new DeviceParamEntity();
		deviceParamEntity.setParamId((long)paramId);
		deviceParamEntity.setParamName(paramName);
		deviceParamEntity.setParamBaudrate((long)paramBaudrate);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		DeviceParamEntity deviceParamEntity2 = null;
		try {
			deviceParamEntity2 = deviceParamService.queryDeviceParamsById(deviceParamEntity);
			if (deviceParamEntity2 != null) {
				if (paramName.equalsIgnoreCase(deviceParamEntity2.getParamName())) {
					object = JSON.toJSON(jsonPackUtils.error("修改失败，重复参数名称"));
				} else {
					deviceParamService.updateDeviceParam(deviceParamEntity);
					object = JSON.toJSON(jsonPackUtils.ok("修改参数信息成功"));
					System.out.println(JSON.toJSON(object));
				}
			} else {
				System.out.println(JSON.toJSON(object));
				object = JSON.toJSON(jsonPackUtils.error(deviceParamEntity.getParamName() + "不存在"));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("修改失败，重复参数名称"));
			e.printStackTrace();
		}
		return object;
	}
	
	// 删除参数信息资料，根据paramId
	@RequestMapping("/delete")
	public <T> Object deleteDeviceParam(@RequestParam(value = "paramId", required = false) Integer paramId) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		DeviceParamEntity deviceParamEntity = new DeviceParamEntity();
		deviceParamEntity.setParamId((long) paramId);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		DeviceParamEntity DeviceParamEntity1 = null;
		try {
			DeviceParamEntity1 = deviceParamService.queryDeviceParamsById(deviceParamEntity);

			if (DeviceParamEntity1 != null) {
				deviceParamService.deleteDeviceParamById(DeviceParamEntity1);
				object = JSON.toJSON(jsonPackUtils.ok("删除成功"));
				System.out.println(JSON.toJSON(object));
			} else {
				System.out.println(JSON.toJSON(object));
				object = JSON.toJSON(jsonPackUtils.error("不存在当前参数"));

			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			e.printStackTrace();
		}
		return object;
	}
}
