package com.mh.eems.controller;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson.JSONObject;
import com.mh.eems.entity.*;
import com.mh.eems.service.DeviceInstallService;
import com.mh.eems.service.UserService;
import com.mh.eems.util.DateUtil;
import org.apache.catalina.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.metadata.Sheet;
import com.alibaba.fastjson.JSON;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.service.ExchangeDeviceService;
import com.mh.eems.util.JsonPackUtils;
import com.mh.eems.util.StringUtil;

/**
 * @ClassName: ExchangeController
 * @Description:TODO(表具更换管理页面接口)
 * @author: 铭汉科技—LJF
 * @date: 2019年11月19日 下午1:57:44
 */
@RestController
@RequestMapping("/exchange")
public class ExchangeController {

    @Autowired
    private ExchangeDeviceService exchangeDeviceService;
    @Autowired
    UserService userService;
    @Autowired
    DeviceInstallService deviceInstallService;


    // 查询全部资料
    // 根据条件查询对应的基表信息
    @LogAnnotation(module = "表具更换管理", logs = "根据条件查询符合内容的表具更换信息内容", type = "查询")
    @RequestMapping("/queryByType")        // update by ljf on 2019-09-29 导出查询结果集的全部基表信息，传page、limit为空
    public <T> Object queryByType(@RequestParam(value = "page", required = false) Integer page,
                                  @RequestParam(value = "limit", required = false) Integer limit,
                                  @RequestParam(value = "houseNum", required = false) String houseNum,
                                  @RequestParam(value = "buildingName", required = false) String buildingName,
                                  @RequestParam(value = "paramType", required = false) String paramType,
                                  @RequestParam(value = "dataCom", required = false) String dataCom,
                                  @RequestParam(value = "updateTime", required = false) String updateTime) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        ExchangeDeviceEntity exchangeDeviceEntity = new ExchangeDeviceEntity();
        if (houseNum != null && houseNum != "") {
            exchangeDeviceEntity.setHouseNum(houseNum);
        }
        if (buildingName != null && buildingName != "") {
            exchangeDeviceEntity.setBuildingName(buildingName);
        }
        if (paramType != null && paramType != "") {
            exchangeDeviceEntity.setParamType(paramType);
        }
        if (dataCom != null && dataCom != "") {
            exchangeDeviceEntity.setDataCom(dataCom);
        }
        if (updateTime != null && updateTime != "") {
            exchangeDeviceEntity.setUpdateTime(updateTime);
            ;
        }
        List<ExchangeDeviceEntity> list = exchangeDeviceService.queryType(exchangeDeviceEntity, page, limit);
        ResponseEntity<ExchangeDeviceEntity> responseEntity = new ResponseEntity<ExchangeDeviceEntity>();
        responseEntity.setCode(200);
        responseEntity.setMsg("success");
        responseEntity.setData(list);
        responseEntity.setCount(exchangeDeviceService.queryCount(exchangeDeviceEntity, page, limit));
        System.out.println(JSON.toJSON(responseEntity));
        return JSON.toJSON(responseEntity);
    }

    // 添加换表信息
    @PostMapping("/add")
    @ResponseBody
    public <T> Object addExchangeDevice(@RequestBody ExchangeDeviceEntity exchangeDeviceEntity) throws ParseException {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object = null;
        System.out.println(exchangeDeviceEntity.toString());
        UserEntity userEntity = new UserEntity();
        DeviceInstallEntity deviceInstallEntity = new DeviceInstallEntity();
        if (exchangeDeviceEntity.getHouseNum() != null) {
            System.out.println(exchangeDeviceEntity.getHouseNum());
            userEntity.setHouseNum(exchangeDeviceEntity.getHouseNum());
            deviceInstallEntity.setHouseNum(exchangeDeviceEntity.getHouseNum());
        }
        if (exchangeDeviceEntity.getNewDeviceNum() != null) {
            userEntity.setDeviceNum(exchangeDeviceEntity.getNewDeviceNum());
            deviceInstallEntity.setDeviceNum(exchangeDeviceEntity.getNewDeviceNum());
        }
        if (exchangeDeviceEntity.getCollectionNum() != null) {
            userEntity.setCollectionNum(exchangeDeviceEntity.getNewCollectionNum());
            deviceInstallEntity.setCollectionNum(exchangeDeviceEntity.getNewCollectionNum());
        }
        if (exchangeDeviceEntity.getNewValue() != null) {
            deviceInstallEntity.setLastValue(Double.parseDouble(exchangeDeviceEntity.getNewValue()));
        }
        if (exchangeDeviceEntity.getUpdateTime() != null) {
            Date date = DateUtil.getDate(exchangeDeviceEntity.getUpdateTime());
            //userEntity.setUpdateTime(date);
            deviceInstallEntity.setCreateTime(date);
        }
        try {
            exchangeDeviceService.insertExchangeDevice(exchangeDeviceEntity);
            int a = userService.updateUser(userEntity);
            int b = deviceInstallService.updateInstalls(deviceInstallEntity);
            if (a != 0 && b != 0) {
                object = JSON.toJSON(jsonPackUtils.ok("更换表具:" + exchangeDeviceEntity.getNewDeviceNum() + "成功"));
            } else {
                JSON.toJSON(jsonPackUtils.error("更换表具:" + exchangeDeviceEntity.getNewDeviceNum() + "失败，无法同步"));
            }
        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("更换表具" + exchangeDeviceEntity.getNewDeviceNum() + "失败"));
            e.printStackTrace();
        }
        return object;
    }

    /**
     * 添加换表信息
     * 功能更新2019-12-13
     * 1)判断历史表数据，是否进行更新
     * 2)删除wt_now表的相关数据
     *
     * @param <T>
     * @param exchangeDeviceEntity
     * @return
     */
    @PostMapping("/update")
    @ResponseBody
    public <T> Object updateExchangeDevice(@RequestBody ExchangeDeviceEntity exchangeDeviceEntity) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object = null;
        System.out.println(exchangeDeviceEntity.toString());
        try {
            if (exchangeDeviceEntity.getId() != null) {
                exchangeDeviceService.updateExchangeDevice(exchangeDeviceEntity);
                object = JSON.toJSON(jsonPackUtils.ok("更新表具资料" + exchangeDeviceEntity.getNewDeviceNum() + "成功"));
            }
        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("更新表具资料" + exchangeDeviceEntity.getNewDeviceNum() + "失败"));
            e.printStackTrace();
        }
        return object;
    }

    // 删除换表信息
    @PostMapping("/delete")
    public <T> Object deleteExchangeDevice(@RequestParam("id") Long id) {
        DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        try {
            exchangeDeviceService.deleteExchangeDevice(id);
            object = JSON.toJSON(jsonPackUtils.ok("删除更换资料成功"));
        } catch (Exception e) {
            // TODO: handle exception
            object = JSON.toJSON(jsonPackUtils.error("删除更换资料失败"));
            e.printStackTrace();
        }
        return object;
    }

    // 通过excel批量导入数据
    @RequestMapping("/importExp")
    @ResponseBody
    public <T> Object importExp(@RequestParam("file") MultipartFile file, HttpServletRequest request) {
        // 判断文件是否为空
        JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
        Object object;
        object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
        if (!file.isEmpty()) {
            try {
                String originalFilename = file.getOriginalFilename();//原文件名字
                System.out.println("文件名：" + originalFilename);
                InputStream inputStream = file.getInputStream();//获取输入流
                BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
                DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
                List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2ExchangeDeviceEntity.class));        // 第几张工作表，第几行开始，javamodel
                inputStream.close();
                ExchangeDeviceEntity exchangeDeviceEntity = new ExchangeDeviceEntity();
                // 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
                int resultInt = 0;
                int batchCount = 150;// 每批commit的个数
                int batchLastIndex = batchCount;// 每批最后一个的下标
                for (int index = 0; index < data.size(); ) {
                    if (batchLastIndex >= data.size()) {
                        batchLastIndex = data.size();
                        //   resultInt = resultInt * exchangeDeviceService.batchInsertDevice(data.subList(index, batchLastIndex));
                        System.out.println();
                        String json = JSON.toJSONString(data.get(index));
                        exchangeDeviceEntity = JSON.parseObject(json, ExchangeDeviceEntity.class);
                        object = addExchangeDevice(exchangeDeviceEntity);

                        System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
                        break;// 数据插入完毕，退出循环
                    } else {
                        //    resultInt = resultInt * exchangeDeviceService.batchInsertDevice(data.subList(index, batchLastIndex));
                        String json = JSON.toJSONString(data.get(index));
                        exchangeDeviceEntity = JSON.parseObject(json, ExchangeDeviceEntity.class);
                        object = addExchangeDevice(exchangeDeviceEntity);
                        System.out.println("index:" + index + " batchLastIndex:" + batchLastIndex);
                        index = batchLastIndex;// 设置下一批下标
                        batchLastIndex = index + (batchCount - 1);
                    }
                }
                System.out.println("批量插入长度：" + data.size());
//			        int a = userService.batchInsert(data);
                if (batchLastIndex== data.size()) {
                    object = JSON.toJSON(jsonPackUtils.ok("导入" + originalFilename + "成功"));
                }else {
                    object = JSON.toJSON(jsonPackUtils.ok("导入" + originalFilename + "失败，无法同步"));
                }
                StringUtil stringUtil = new StringUtil();
                //  stringUtil.print(data.);
            } catch (Exception e) {
                object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
                e.printStackTrace();
            }
        }
        return object;
    }

}
