package com.mh.eems.controller;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.util.List;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.metadata.Sheet;
import com.mchange.v2.management.ManagementUtils;
import com.mh.eems.entity.*;
import com.mh.eems.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.dto.GatewayStatusDto;
import com.mh.eems.service.GatewayService;
import com.mh.eems.util.JsonPackUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;



/**
 * 
 * @ClassName:  GatewayController   
 * @Description:TODO(网关前端接口)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年8月1日 上午8:29:21   
 *
 */
@RestController
@RequestMapping("/gateway")
public class GatewayController {
	
	@Autowired
	private GatewayService gatewayService;
	/**
	 *通过Excel导入增加网关信息
	 * 2020年5月8日
	 * @param file
	 * @param <T>
	 * @return
	 */
// 通过excel批量导入数据
	@RequestMapping("/importExp")
	@ResponseBody
	public <T> Object importExp(@RequestParam("file") MultipartFile file, HttpServletRequest request){
		// 判断文件是否为空
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
		if (!file.isEmpty()) {
			try {
				String originalFilename = file.getOriginalFilename();//原文件名字
				System.out.println("文件名："+originalFilename);
				InputStream inputStream = file.getInputStream();//获取输入流
				BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
				DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
				List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2GatewayInstallEntity.class));		// 第几张工作表，第几行开始，javamodel
				inputStream.close();
				// 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
				int resultInt = 0;
				int batchCount = 150;// 每批commit的个数
				int batchLastIndex = batchCount;// 每批最后一个的下标
				for (int index = 0; index < data.size();) {
					if (batchLastIndex >= data.size()) {
						batchLastIndex = data.size();
						resultInt = resultInt * gatewayService.GatewayBatchInsert(data.subList(index, batchLastIndex));

						System.out.println("index:" + index+ " batchLastIndex:" + batchLastIndex);
						break;// 数据插入完毕，退出循环
					} else {
						resultInt = resultInt * gatewayService.GatewayBatchInsert(data.subList(index, batchLastIndex));
						System.out.println("index:" + index+ " batchLastIndex:" + batchLastIndex);
						index = batchLastIndex;// 设置下一批下标
						batchLastIndex = index + (batchCount - 1);
					}
				}
				System.out.println("批量插入长度："+data.size());
//		        int a = userService.batchInsert(data);
				if (batchLastIndex == data.size()) {
					object = JSON.toJSON(jsonPackUtils.ok("导入"+originalFilename+"成功"));
				}
				StringUtil stringUtil = new StringUtil();
				stringUtil.print(data);
			} catch (Exception e) {
				object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
				e.printStackTrace();
			}
		}
		return object;
	}
	
	// 查询全部网关信息
	@LogAnnotation(module="网关管理", logs = "查询全部网关设备", type = "查询")
	@GetMapping("/queryAll")
	public Object queryAllGateway(@RequestParam(value="page", required=false)Integer page,
			@RequestParam(value="limit", required=false)Integer limit) {
		Object object = null;
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<GateWayManageEntity> responseEntity = new ResponseEntity<GateWayManageEntity>();
		GateWayManageEntity gm = new GateWayManageEntity();
		try {
			List<GateWayManageEntity> gateWayManageEntities = gatewayService.queryGatewayByOther(gm, page, limit,null,null);
			System.out.println(JSON.toJSON(gateWayManageEntities));
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(gateWayManageEntities);
		} catch (Exception e) {
			// TODO: handle exception
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(null);
			responseEntity.setCount(0);
			object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
				      SerializerFeature.WriteMapNullValue);
			e.printStackTrace();
			return object;
		}
		responseEntity.setCount(gatewayService.queryGatewayCount(gm, page, limit,null,null));
		object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue);
		System.out.println(JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue));
		return object;
	}
	
	// 根据条件查询网关信息
	@LogAnnotation(module="网关管理", logs = "按条件查询网关设备", type = "查询")
	@GetMapping("/query")
	public Object queryGateway(@RequestParam(value="page", required=false)Integer page,
			@RequestParam(value="limit", required=false)Integer limit,
			@RequestParam(value="gatewayId", required=false)Integer gatewayId,
			@RequestParam(value="gatewayName", required=false)String gatewayName,
			@RequestParam(value="dataCom", required=false)String dataCom,
			@RequestParam(value="status", required=false)String status,
			@RequestParam(value="start", required=false)String start,
			@RequestParam(value="end", required=false)String end,
			@RequestParam(value="internetCard",required = false)String internetCard,
			@RequestParam(value="operator",required = false)String operator) {
		Object object = null;
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<GateWayManageEntity> responseEntity = new ResponseEntity<GateWayManageEntity>();
		GateWayManageEntity gm = new GateWayManageEntity();
		if (gatewayId != null) {
			gm.setGatewayId((long)gatewayId);
		}
		if (gatewayName != null || gatewayName != "") {
			gm.setGatewayName(gatewayName);
		}
		if (dataCom != null || dataCom != "") {
			gm.setDataCom(dataCom);
		}
		if (status != null && status != "") {
			if (status.equals("正常")) {
				gm.setStatus(1);
			} else if (status.equals("异常")) {
				gm.setStatus(2);
			} else if (status.equals("离线")) {
				gm.setStatus(0);
			}
			
		}
		if(internetCard != null || internetCard != "") {
			gm.setInternetCard(internetCard);
		}
		System.out.println(operator);
		if(operator != null && operator != "") {
			if(operator.equals("中国移动")) {
				gm.setOperator(0);
			}else if (operator.equals("中国联通")) {
				gm.setOperator(1);
			}else if(operator.equals("中国电信")) {
				gm.setOperator(2);
			}
		}
		try {
			List<GateWayManageEntity> gateWayManageEntities = gatewayService.queryGatewayByOther(gm, page, limit, start, end);
			System.out.println(JSON.toJSON(gateWayManageEntities));
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(gateWayManageEntities);
		} catch (Exception e) {
			// TODO: handle exception
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(null);
			responseEntity.setCount(0);
			object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
				      SerializerFeature.WriteMapNullValue);
			e.printStackTrace();
			return object;
		}
		responseEntity.setCount(gatewayService.queryGatewayCount(gm, page, limit,start,end));
		object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue);
		System.out.println(JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue));
		return object;
	}
	
	/**
	 * 增加网关信息
	 * 2019-12-20 增加插入字段 
	 * @param <T>
	 * @param gateWayManageEntity
	 * @return
	 */
	@PostMapping("/insert")
	@ResponseBody
	public <T> Object addGateway(@RequestBody GateWayManageEntity gateWayManageEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
//		gateWayManageEntity.setGatewayId((long)5);
//		gateWayManageEntity.setGatewayName("test");
//		gateWayManageEntity.setGatewayAddress("test");
//		gateWayManageEntity.setDataCom("COM5");
		
		String result = gatewayService.addGateway(gateWayManageEntity);
		
		if (result.equals("success")) {
			object = JSON.toJSON(jsonPackUtils.ok("添加网关"+gateWayManageEntity.getGatewayId()+"成功")); 
		} else {
			object = JSON.toJSON(jsonPackUtils.error(result)); 
		}
		System.out.println(object);
		return object;
	}
	
	/**
	 * 修改网关信息
	 * 2019-12-20 增加update的字段
	 * @param <T>
	 * @param gateWayManageEntity
	 * @return
	 */
	@PostMapping("/update")
	@ResponseBody
	public <T> Object updateGateway(@RequestBody GateWayManageEntity gateWayManageEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
//		gateWayManageEntity.setGatewayId((long)5);
//		gateWayManageEntity.setGatewayName("test");
//		gateWayManageEntity.setGatewayAddress("test");
//		gateWayManageEntity.setDataCom("COM5");
		
		String result = gatewayService.updateGateway(gateWayManageEntity);
		
		if (result.equals("success")) {
			object = JSON.toJSON(jsonPackUtils.ok("更新网关"+gateWayManageEntity.getGatewayId()+"成功")); 
		} else {
			object = JSON.toJSON(jsonPackUtils.error(result)); 
		}
		System.out.println(object);
		return object;
	}
	
	// 根据网关ID删除网关
	@GetMapping("/delete")
	public <T> Object deleteGateway(@RequestParam(value = "gatewayId", required = false)Integer gatewayId) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		
		int result = gatewayService.deleteGaway(gatewayId);
		
		if (result != 0) {
			object = JSON.toJSON(jsonPackUtils.ok("删除记录成功")); 
		} else {
			object = JSON.toJSON(jsonPackUtils.error("删除记录失败")); 
		}
		System.out.println(object);
		return object;
	}
	
	// 首页页面的网关状态饼图
	@GetMapping("/home")
	public Object queryAllHome() {
		Object object = null;
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<GatewayStatusDto> responseEntity = new ResponseEntity<GatewayStatusDto>();
		try {
			List<GatewayStatusDto> gatewayStatusDtos = gatewayService.queryStatusForGateway();
			System.out.println(JSON.toJSON(gatewayStatusDtos));
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(gatewayStatusDtos);
		} catch (Exception e) {
			// TODO: handle exception
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(null);
			responseEntity.setCount(0);
			object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
				      SerializerFeature.WriteMapNullValue);
			e.printStackTrace();
			return object;
		}
		object = JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue);
		System.out.println(JSON.toJSONString(responseEntity, SerializerFeature.PrettyFormat, 
			      SerializerFeature.WriteMapNullValue));
		return object;
	}

}
