package com.mh.eems.controller;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.map.HashedMap;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.JSONSerializable;
import com.alibaba.fastjson.serializer.JSONSerializer;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.dto.Token;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.SysUserEntity;
import com.mh.eems.service.TokenManager;
import com.mh.eems.util.FastJsonUtil;
import com.mh.eems.util.SysUserUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @ClassName:  LoginController   
 * @Description:TODO(登录)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年6月14日 下午5:01:30   
 *
 */
@Api(tags = "登陆")
@RestController
@RequestMapping
public class LoginController {

	@Autowired
	private TokenManager tokenManager;
	@Autowired
	private ServerProperties serverProperties;

	@LogAnnotation(module="web端登录", logs = "web端登录", type = "查询")
	@ApiOperation(value = "web端登陆")
	@PostMapping("/sys/login")
//	@CrossOrigin
	public Object login(String username, String password) {
	//		DBIdentifier.setProjectCode("网络电表");
		ResponseEntity<?> responseEntity = new ResponseEntity();
//		List<Object> objects = new ArrayList<Object>();
		try {
			UsernamePasswordToken usernamePasswordToken = new UsernamePasswordToken(username, password);
			SecurityUtils.getSubject().login(usernamePasswordToken);
			// 设置shiro的session过期时间
			SecurityUtils.getSubject().getSession().setTimeout(serverProperties.getServlet().getSession().getTimeout().toMillis());
			responseEntity.setCode(200);
			responseEntity.setMsg("登录成功");
			responseEntity.setData(null);
//			responseEntity.setData(objects);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			responseEntity.setCode(200);
			responseEntity.setMsg(e.getMessage());
			responseEntity.setData(null);
//			responseEntity.setData(objects);
		}
		System.out.println(JSONObject.toJSON(responseEntity));
		return JSONObject.toJSON(responseEntity);
	}

	@LogAnnotation
	@ApiOperation(value = "Restful方式登陆,前后端分离时登录接口")
	@PostMapping("/sys/login/restful")
	public Token restfulLogin(String username, String password) {
//		DBIdentifier.setProjectCode("网络电表");
		UsernamePasswordToken usernamePasswordToken = new UsernamePasswordToken(username, password);
		SecurityUtils.getSubject().login(usernamePasswordToken);

		return tokenManager.saveToken(usernamePasswordToken);
	}

	@ApiOperation(value = "当前登录用户")
	@GetMapping("/sys/login")
	public SysUserEntity getLoginInfo() {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		return SysUserUtil.getCurrentUser();
	}

}
