package com.mh.eems.controller;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.entity.PermissionEntity;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.SysUserEntity;
import com.mh.eems.mapper.PermissionMapper;
import com.mh.eems.service.PermissionService;
import com.mh.eems.util.SysUserUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import javassist.bytecode.CodeIterator.Gap;

/**
 * 权限相关接口
 * 
 * @author 小威老师 xiaoweijiagou@163.com
 *
 */
@Api(tags = "权限")
@RestController
@RequestMapping("/permissions")
public class PermissionController {

	@Autowired
	private PermissionMapper permissionMapper;
	@Autowired
	private PermissionService permissionService;

	@Autowired
	private ServerProperties serverProperties;
	
	@LogAnnotation(module="web端登录", logs = "当前登录用户拥有的权限", type = "查询")
	@ApiOperation(value = "当前登录用户拥有的权限")
	@GetMapping("/current")
	public Object permissionsCurrent() {

		ResponseEntity<PermissionEntity> responseEntity = new ResponseEntity<PermissionEntity>();
		try {
			List<PermissionEntity> list = SysUserUtil.getCurrentPermissions();
			if (list == null) {
				
//				// test
//				UsernamePasswordToken usernamePasswordToken = new UsernamePasswordToken("admin", "admin");
//				SecurityUtils.getSubject().login(usernamePasswordToken);
//				// 设置shiro的session过期时间
//				SecurityUtils.getSubject().getSession().setTimeout(serverProperties.getServlet().getSession().getTimeout().toMillis());
//				// test
				
				SysUserEntity user = SysUserUtil.getCurrentUser();
				DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
				list = permissionService.listByUserId(user.getId());
//				UserUtil.setPermissionSession(list);
			}
			final List<PermissionEntity> permissions = list.stream().filter(l -> l.getType().equals(1))
					.collect(Collectors.toList());
			List<PermissionEntity> firstLevel = permissions.stream().filter(p -> p.getParentId().equals(0L)).collect(Collectors.toList());
			firstLevel.parallelStream().forEach(p -> {
				setChild(p, permissions);
			});
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
//			responseEntity.setData(list);
			responseEntity.setData(firstLevel);
		}catch (Exception e) {
			e.printStackTrace();
			responseEntity.setCode(201);
			responseEntity.setMsg(e.getMessage());
			responseEntity.setData(null);
		}
		System.out.println(JSONObject.toJSON(responseEntity));
		return JSONObject.toJSON(responseEntity);
	}

	/**
	 * 设置子元素
	 * 2018.06.09
	 *
	 * @param p
	 * @param permissions
	 */
	private void setChild(PermissionEntity p, List<PermissionEntity> permissions) {
		System.out.println(p.getId());
		List<PermissionEntity> child = permissions.parallelStream().filter(a -> a.getParentId().equals(p.getId())).collect(Collectors.toList());
		p.setChild(child);
		if (!CollectionUtils.isEmpty(child)) {
			child.parallelStream().forEach(c -> {
				//递归设置子元素，多级菜单支持
				setChild(c, permissions);
			});
		}
	}

//	private void setChild(List<Permission> permissions) {
//		permissions.parallelStream().forEach(per -> {
//			List<Permission> child = permissions.stream().filter(p -> p.getParentId().equals(per.getId()))
//					.collect(Collectors.toList());
//			per.setChild(child);
//		});
//	}

	/**
	 * 菜单列表
	 * 
	 * @param pId
	 * @param permissionsAll
	 * @param list
	 */
	private void setPermissionsList(Long pId, List<PermissionEntity> permissionsAll, List<PermissionEntity> list) {
		for (PermissionEntity per : permissionsAll) {
			if (per.getParentId().equals(pId)) {
				list.add(per);
				if (permissionsAll.stream().filter(p -> p.getParentId().equals(per.getId())).findAny() != null) {
					setPermissionsList(per.getId(), permissionsAll, list);
				}
			}
		}
	}

	@GetMapping
	@ApiOperation(value = "菜单列表")
	@RequiresPermissions("sys:menu:query")
	public List<PermissionEntity> permissionsList() {
		List<PermissionEntity> permissionsAll = permissionMapper.listAll();

		List<PermissionEntity> list = Lists.newArrayList();
		setPermissionsList(0L, permissionsAll, list);

		return list;
	}

	@GetMapping("/all")
	@ApiOperation(value = "所有菜单")
	@RequiresPermissions("sys:menu:query")
	public JSONArray permissionsAll() {
		DBIdentifier.setProjectCode("网络电表");
		List<PermissionEntity> permissionsAll = permissionMapper.listAll();
		JSONArray array = new JSONArray();
		setPermissionsTree(0L, permissionsAll, array);

		return array;
	}

	@GetMapping("/parents")
	@ApiOperation(value = "一级菜单")
	@RequiresPermissions("sys:menu:query")
	public List<PermissionEntity> parentMenu() {
		DBIdentifier.setProjectCode("网络电表");
		List<PermissionEntity> parents = permissionMapper.listParents();

		return parents;
	}

	/**
	 * 菜单树
	 * 
	 * @param pId
	 * @param permissionsAll
	 * @param array
	 */
	private void setPermissionsTree(Long pId, List<PermissionEntity> permissionsAll, JSONArray array) {
		for (PermissionEntity per : permissionsAll) {
			if (per.getParentId().equals(pId)) {
				String string = JSONObject.toJSONString(per);
				JSONObject parent = (JSONObject) JSONObject.parse(string);
				array.add(parent);

				if (permissionsAll.stream().filter(p -> p.getParentId().equals(per.getId())).findAny() != null) {
					JSONArray child = new JSONArray();
					parent.put("child", child);
					setPermissionsTree(per.getId(), permissionsAll, child);
				}
			}
		}
	}

	@GetMapping(params = "roleId")
	@ApiOperation(value = "根据角色id删除权限")
	@RequiresPermissions(value = { "sys:menu:query", "sys:role:query" }, logical = Logical.OR)
	public List<PermissionEntity> listByRoleId(Long roleId) {
		return permissionMapper.listByRoleId(roleId);
	}

	@LogAnnotation
	@PostMapping
	@ApiOperation(value = "保存菜单")
	@RequiresPermissions("sys:menu:add")
	public void save(@RequestBody PermissionEntity permission) {
		permissionMapper.save(permission);
	}

	@GetMapping("/{id}")
	@ApiOperation(value = "根据菜单id获取菜单")
	@RequiresPermissions("sys:menu:query")
	public PermissionEntity get(@PathVariable Long id) {
		return permissionMapper.getById(id);
	}

	@LogAnnotation
	@PutMapping
	@ApiOperation(value = "修改菜单")
	@RequiresPermissions("sys:menu:add")
	public void update(@RequestBody PermissionEntity permission) {
		permissionMapper.update(permission);
	}

	/**
	 * 校验权限
	 * 
	 * @return
	 */
	@GetMapping("/owns")
	@ApiOperation(value = "校验当前用户的权限")
	public Set<String> ownsPermission() {
		List<PermissionEntity> permissions = SysUserUtil.getCurrentPermissions();
		if (CollectionUtils.isEmpty(permissions)) {
			return Collections.emptySet();
		}

		return permissions.parallelStream().filter(p -> !StringUtils.isEmpty(p.getPermission()))
				.map(PermissionEntity::getPermission).collect(Collectors.toSet());
	}

	@LogAnnotation
	@DeleteMapping("/{id}")
	@ApiOperation(value = "删除菜单")
	@RequiresPermissions(value = { "sys:menu:del" })
	public void delete(@PathVariable Long id) {
		permissionService.delete(id);
	}
}
