package com.mh.eems.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.dto.RolesDto;
import com.mh.eems.dto.RoleAndPermissionDto;
import com.mh.eems.entity.PermissionEntity;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.ResponseInfoEntity;
import com.mh.eems.entity.RoleEntity;
import com.mh.eems.service.RoleService;
import com.mh.eems.util.JsonPackUtils;

import io.swagger.annotations.Api;
import net.sf.ehcache.search.expression.And;

/**
 * 
 * @ClassName:  RoleManageController   
 * @Description:TODO(系统管理模块：角色管理接口)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年7月24日 上午8:58:17   
 *
 */
@Api(tags = "查询角色")
@RestController
@RequestMapping("/role")
public class RoleManageController {
	
	@Autowired
	private RoleService roleService;
	
	// 查询全部
	@LogAnnotation(module="角色管理", logs = "系统角色查询", type = "查询")
	@GetMapping("/queryAllRoles")
	public Object queryAllRoles(@RequestParam(value="page", required=false)Integer page,
			@RequestParam(value="limit", required=false)Integer limit) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<RoleEntity> responseEntity = new ResponseEntity<RoleEntity>();
		List<RoleEntity> roleEntities = null;
		if ((page != null) && (limit != null)) {
			roleEntities = roleService.queryAllRoles(page,limit);
		} else {
			roleEntities = roleService.queryAllRoles1();
		}
		
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(roleEntities);
		responseEntity.setCount(roleService.queryRoleCount());
		return JSON.toJSON(responseEntity);
	}
	
	// 通过ID查询
	@LogAnnotation(module="角色管理", logs = "通过角色ID查询", type = "查询")
	@GetMapping("/queryRoleById")
	public Object queryRoleById(int roleId) {
//		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
//		ResponseEntity<RoleAndPermissionDto> responseEntity = new ResponseEntity<RoleAndPermissionDto>();
//		List<RoleAndPermissionDto> testDto = new ArrayList<RoleAndPermissionDto>();
////		testDto = roleService.queryRoleAndPermission((long) roleId);
//		responseEntity.setData(testDto);
//		responseEntity.setCode(200);
//		responseEntity.setMsg("success");
//		responseEntity.setCount(0);
//		System.out.println(JSON.toJSON(responseEntity));
//		return JSON.toJSON(responseEntity);
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseInfoEntity responseEntity = new ResponseInfoEntity();
		RolesDto responseInfo1 = new RolesDto();
		List<RoleAndPermissionDto> testDto = new ArrayList<RoleAndPermissionDto>();
		testDto = roleService.queryRoleAndPermission((long)roleId);
		System.out.println();
		responseInfo1.setPermissions(JSON.toJSON(testDto.get(0).getPermissions()));
		responseInfo1.setRoles(JSON.toJSON(testDto.get(0).getRoles()));
		responseEntity.setData(JSON.toJSON(responseInfo1));
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setCount(0);
		System.out.println(JSON.toJSON(responseEntity));
		return JSON.toJSON(responseEntity);
	}
	
	// 插入角色
	@GetMapping("/insertRole")
	public <T> Object insertRole(String roleName,String desc) {
		Object object = null;
		RoleEntity roleEntity = new RoleEntity();
		roleEntity.setRoleName(roleName);
		roleEntity.setDesc(desc);
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		// 判断是否存在当前角色
		RoleEntity count = roleService.queryRoleByRoleName(roleEntity);
		if (count == null) {
			roleService.insertRole(roleEntity);
			Long a = roleEntity.getRoleId();
			if (a != 0) {
				// 插入角色和权限id，sys_role_permission
				roleService.insertRoleAndPermisson();
				// 插入角色和用户之间的关联性，sys_role_user
				roleService.InsertRoleAndUser();
				JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
				object = JSON.toJSON(jsonPackUtils.ok("添加"+roleName+"成功")); 
				System.out.println(JSON.toJSON(object));
			}
		} else {
			JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
			object = JSON.toJSON(jsonPackUtils.error("已存在当前角色")); 
			System.out.println(JSON.toJSON(object));
		}
		return object;
	}
	
	// 根据ID删除角色
	@GetMapping("/deleteRoleById")
	public <T> Object deleteRoleById(int roleId) {
		Object object = null;
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		int a = roleService.deleteRoleById(roleId);
		if (a != 0) {
			// 删除角色和权限id，sys_role_permission
			roleService.deleteRolePermissionById(roleId);
			JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
			object = JSON.toJSON(jsonPackUtils.ok("删除成功")); 
			System.out.println(JSON.toJSON(object));
		}
		return object;
	}
	
	// 更新某个角色对应的权限
	@PostMapping("/updateRole")
	@ResponseBody
	public <T> Object updateRoleAndPermission(@RequestBody String receiveData) {
		Object object = null;
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		// 获取对应的sys_role值，然后进行更新
		RoleEntity roleEntity = new RoleEntity();
		// 解析receiveData
		JSONObject jsonObject =  JSONObject.parseObject(receiveData);
		System.out.println("接收到的值："+receiveData);
		roleEntity.setRoleId(jsonObject.getLongValue("roleId"));
		roleEntity.setRoleName(jsonObject.getString("roleName"));
		roleEntity.setDesc(jsonObject.getString("desc"));
		// 更新sys_role值
		try {
			int a = roleService.updateRoleById(roleEntity);
			if (a != 0) {
				List<PermissionEntity> pList = new ArrayList<PermissionEntity>();
				JSONArray jsonArray = jsonObject.getJSONArray("permissions");
				for (int i = 0; i < jsonArray.size(); i++) {
					PermissionEntity permission = new PermissionEntity();
					JSONObject jsonObject2 = jsonArray.getJSONObject(i);
					permission.setName(jsonObject2.getString("name"));
					permission.setId(jsonObject2.getLong("id"));
					permission.setIsStatus(jsonObject2.getInteger("isStatus"));
					permission.setSort(roleEntity.getRoleId().intValue());
					pList.add(permission);
				}
				// 更新sys_role_permission
				roleService.updateRolePermission(pList);
				
				JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
				object = JSON.toJSON(jsonPackUtils.ok("更新"+roleEntity.getRoleName()+"成功")); 
				System.out.println(JSON.toJSON(object));
			}
		} catch (Exception e) {
			// TODO: handle exception
			JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
			object = JSON.toJSON(jsonPackUtils.error("更新出错，角色名称重复"));
			e.printStackTrace();
		}
		return object;
	}
	
}
