package com.mh.eems.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.dto.UserAndRoleDto;
import com.mh.eems.dto.UserDto;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.SysUserEntity;
import com.mh.eems.service.SysUserService;
import com.mh.eems.util.JsonPackUtils;

import io.swagger.annotations.Api;

/**
 * 
 * @ClassName:  SysUserController   
 * @Description:TODO(查询系统用户)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年7月26日 上午9:35:57   
 *
 */
@Api(tags = "系统用户管理")
@RestController
@RequestMapping("/sysUser")
public class SysUserController {
	
	@Autowired
	private SysUserService userService;
	
	// 更新用户
//	@LogAnnotation(module="系统用户管理", logs = "系统用户更新", type = "更新")
	@PostMapping("/update")
	public <T> Object updateUser(@RequestBody UserDto userDto) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		try {
				userService.updateUser(userDto);
				object = JSON.toJSON(jsonPackUtils.ok("更新"+userDto.getUserName()+"成功")); 
				System.out.println(JSON.toJSON(object));
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("不能相同用户名，请重新输入")); 
			System.out.println(JSON.toJSON(object));
		}
		return object;
	}
	
	// 删 除用户，以及删除对应的sys_role_user关联值
//	@LogAnnotation(module="用户管理", logs = "系统用户删除", type = "删除")
	@RequestMapping("/delete")
	public <T> Object deleteSysUser(int userId) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		try {
			userService.deleteUser((long)userId);
			object = JSON.toJSON(jsonPackUtils.ok("删除成功")); 
			System.out.println(JSON.toJSON(object));
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			e.printStackTrace();
		}
		return object;
	}
	
	// 更新用户状态
	@RequestMapping("/updateStatus")
	public <T> Object updateStatus(int userId, int status) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		try {
			userService.updateStatus(userId, status);
			object = JSON.toJSON(jsonPackUtils.ok("更新状态成功")); 
			System.out.println(JSON.toJSON(object));
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("更新状态失败"));
			e.printStackTrace();
		}
		return object;
	}
	 
	
	// 添加用户
	@PostMapping("/add")
	public <T> Object addUser(@RequestBody UserDto userDto) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		Object object = null;
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		SysUserEntity u = userService.getUser(userDto.getUserName());
		try {
			if (u != null) {
				
				throw new IllegalArgumentException(userDto.getUserName() + "已存在");
			} else {
				userService.saveUser(userDto);
				object = JSON.toJSON(jsonPackUtils.ok("添加"+userDto.getUserName()+"成功")); 
				System.out.println(JSON.toJSON(object));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error(e.getMessage())); 
			System.out.println(JSON.toJSON(object));
		}
		return object;
	}
	
	// 查询全部用户
	@LogAnnotation(module="系统用户管理", logs = "查询全部系统用户", type = "查询")
	@GetMapping("/queryAllUsers")
	public Object queryAllUsers(int page,int limit) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<SysUserEntity> responseEntity = new ResponseEntity<SysUserEntity>();
		List<SysUserEntity> users = userService.queryAllUsers(page,limit);
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(users);
		responseEntity.setCount(userService.queryUserCount());
		return JSON.toJSON(responseEntity);
	}
	
	// 根据用户ID查询用户以及拥有角色类型
	@GetMapping("/queryUserById")
	public Object queryUserById(int userId) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		ResponseEntity<UserAndRoleDto> responseEntity = new ResponseEntity<UserAndRoleDto>();
		try {
			List<UserAndRoleDto> users = userService.querySysUserAndRoleByUserId(userId);
			responseEntity.setCode(200);
			responseEntity.setMsg("success");
			responseEntity.setData(users);
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			responseEntity.setCode(200);
			responseEntity.setMsg("fail");
			responseEntity.setData(null);
		}
		System.out.println(JSONObject.toJSON(responseEntity));
		return JSON.toJSON(responseEntity);
	}

}
