package com.mh.eems.controller;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.metadata.Sheet;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.constants.UserConstants;
import com.mh.eems.datasource.DBIdentifier;
import com.mh.eems.entity.Excel2UserEntity;
import com.mh.eems.entity.ResponseEntity;
import com.mh.eems.entity.UserEntity;
import com.mh.eems.entity.UserTreeEntity;
import com.mh.eems.service.UserService;
import com.mh.eems.util.JsonPackUtils;
import com.mh.eems.util.StringUtil;


/**'
 * 
 * @ClassName:  UserController   
 * @Description:TODO(用户管理前端接口)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年8月5日 上午8:39:59   
 *
 */
@RestController
@RequestMapping("/user")
public class UserController {

	@Autowired
	private UserService userService;
	

	// 通过excel批量导入数据
	@RequestMapping("/importExp")
	@ResponseBody
	public <T> Object importExp(@RequestParam("file") MultipartFile file, HttpServletRequest request){
		// 判断文件是否为空
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
		if (!file.isEmpty()) {
			try {
				String originalFilename = file.getOriginalFilename();//原文件名字
				System.out.println("文件名："+originalFilename);
				InputStream inputStream = file.getInputStream();//获取输入流
				BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
				DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		        List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2UserEntity.class));		// 第几张工作表，第几行开始，javamodel
		        inputStream.close();
		        // 避免出现RPC 请求中提供了过多的参数。最多应为 2100报错，循环分批次插入update by ljf on 2019-08-23
		        int resultInt = 0;
				int batchCount = 250;// 每批commit的个数
		        int batchLastIndex = batchCount;// 每批最后一个的下标
		        for (int index = 0; index < data.size();) {
		            if (batchLastIndex >= data.size()) {
		                batchLastIndex = data.size();
		                resultInt = resultInt * userService.batchInsert(data.subList(index, batchLastIndex));
		                System.out.println("index:" + index+ " batchLastIndex:" + batchLastIndex);
		                break;// 数据插入完毕，退出循环
		            } else {
		            	resultInt = resultInt * userService.batchInsert(data.subList(index, batchLastIndex));
		                System.out.println("index:" + index+ " batchLastIndex:" + batchLastIndex);
		                index = batchLastIndex;// 设置下一批下标
		                batchLastIndex = index + (batchCount - 1);
		            }
		        }
		        System.out.println("批量插入长度："+data.size());
//		        int a = userService.batchInsert(data);
		        if (batchLastIndex == data.size()) {
		        	object = JSON.toJSON(jsonPackUtils.ok("导入"+originalFilename+"成功"));
				}
		        StringUtil stringUtil = new StringUtil();
		        stringUtil.print(data);
			} catch (Exception e) {
				object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
				e.printStackTrace();
			}
		}
		return object;
	}
	
	// 通过excel批量导入数据
	@RequestMapping("/importExpTest")
	@ResponseBody
	public <T> Object importExpTest(@RequestParam("file") MultipartFile file, HttpServletRequest request){
		// 判断文件是否为空
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
		if (!file.isEmpty()) {
			try {
				String originalFilename = file.getOriginalFilename();//原文件名字
				System.out.println("文件名："+originalFilename);
				InputStream inputStream = file.getInputStream();//获取输入流
				BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
				DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		        List<Object> data = EasyExcelFactory.read(bufferedInputStream, new Sheet(1, 2, Excel2UserEntity.class));		// 第几张工作表，第几行开始，javamodel
		        inputStream.close();
		        StringUtil stringUtil = new StringUtil();
		        stringUtil.print(data);
		        object = JSON.toJSON(jsonPackUtils.ok("导入"+originalFilename+"成功"));
			} catch (Exception e) {
				object = JSON.toJSON(jsonPackUtils.error("导入excel失败，数据重复或者格式错误，请重新导入"));
				e.printStackTrace();
			}
		}
		return object;
	}

	@LogAnnotation(module="系统用户管理", logs = "查询全部系统用户", type = "查询")
	@GetMapping("/queryTrees")
	public <T> Object queryAllUsers() {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		List<UserTreeEntity> list = userService.queryUserTree();
		final List<UserTreeEntity> permissions = list.stream().filter(l -> !l.getLevel().equals(99))
				.collect(Collectors.toList());
		List<UserTreeEntity> firstLevel = permissions.stream().filter(p -> p.getParentId().equals(0L)).collect(Collectors.toList());
		firstLevel.parallelStream().forEach(p -> {
			setChild(p, permissions,1);
		});
		System.out.println(JSON.toJSONString(firstLevel,SerializerFeature.PrettyFormat,SerializerFeature.WriteMapNullValue));
		ResponseEntity<UserTreeEntity> responseEntity = new ResponseEntity<UserTreeEntity>();
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(firstLevel);
		return JSON.toJSON(responseEntity);

	}

	// 增加树节点
	@PostMapping("/addTree")
	@ResponseBody
	public <T> Object addUserTree(@RequestBody UserTreeEntity userTreeEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			int a = userService.insertUserTree(userTreeEntity);
			if (a != 0) {
				object = JSON.toJSON(jsonPackUtils.ok("添加"+userTreeEntity.getTitle()+"成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error("添加"+userTreeEntity.getTitle()+"失败"));
			}

		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("添加"+userTreeEntity.getTitle()+"失败"));
			e.printStackTrace();
		}
		return object;
	}

	// 修改树节点内容
	@PostMapping("/updateTree")
	@ResponseBody
	public <T> Object updateUserTree(@RequestBody UserTreeEntity userTreeEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			int a = userService.updateUserTree(userTreeEntity);
			if (a != 0) {
				object = JSON.toJSON(jsonPackUtils.ok("更新为"+userTreeEntity.getTitle()+"成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error("更新为"+userTreeEntity.getTitle()+"失败"));
			}

		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("更新为"+userTreeEntity.getTitle()+"失败"));
			e.printStackTrace();
		}
		return object;
	}

	// 删除树节点
	@GetMapping("/deleteTree")
	public <T> Object deleteUserTree(@RequestParam(value = "id", required = false) Integer id) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			int a = userService.deleteUserTree(id);
			if (a != 0) {
				object = JSON.toJSON(jsonPackUtils.ok("删除成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			e.printStackTrace();
		}
		return object;
	}

	// 根据树节点的ID值，查询父ID为当前节点Id的值
	@RequestMapping("/queryUsers")
	public Object queryAllUsers(@RequestParam(value = "page",required = false) Integer page,
			@RequestParam(value = "limit",required = false) Integer limit,
			@RequestParam(value = "id",required = true) Integer id,
			@RequestParam(value = "title",required = true) String title) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		UserTreeEntity userTreeEntity = new UserTreeEntity();
		userTreeEntity.setId((long)id);
		userTreeEntity.setTitle(title);
		List<UserEntity> userList = userService.queryUsers(userTreeEntity,page,limit);
		ResponseEntity<UserEntity> responseEntity = new ResponseEntity<UserEntity>();
		System.out.println(JSON.toJSONString(userList,SerializerFeature.PrettyFormat,SerializerFeature.WriteMapNullValue));
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(userList);
		UserEntity userEntity = new UserEntity();
		userEntity.setParentId(userTreeEntity.getId());
		responseEntity.setCount(userService.queryUsersCount(userEntity, page, limit));
		return JSON.toJSON(responseEntity);
	}

	@RequestMapping("/queryUserById")
	public <T> Object queryUserById(@RequestParam(value = "id",required = false) Integer id,
			@RequestParam(value = "title",required = false) String title) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		List<UserEntity> userList = null;
		if (id != null && title != null) {
			userList = userService.queryUserById(title, id);
		}
		List<UserTreeEntity> list = userService.queryUserTree();
		final List<UserTreeEntity> permissions = list.stream().filter(l -> !l.getLevel().equals(99))
				.collect(Collectors.toList());
		List<UserTreeEntity> firstLevel = permissions.stream().filter(p -> p.getParentId().equals(0L)).collect(Collectors.toList());
		firstLevel.parallelStream().forEach(p -> {
			setChild(p, permissions,1);
		});
		System.out.println(JSON.toJSONString(firstLevel,SerializerFeature.PrettyFormat,SerializerFeature.WriteMapNullValue));

		//		ResponseEntity<UserEntity> responseEntity = new ResponseEntity<UserEntity>();
		//		System.out.println(JSON.toJSONString(userList,SerializerFeature.PrettyFormat,SerializerFeature.WriteMapNullValue));
		//		responseEntity.setCode(200);
		//		responseEntity.setMsg("success");
		//		responseEntity.setData(userList);
		//		return JSON.toJSON(responseEntity);
		JSONObject jsonData = new JSONObject();
		jsonData.put("titles", firstLevel);
		jsonData.put("user", userList);

		JSONObject jsonObject = new JSONObject();
		jsonObject.put("code", 200);
		jsonObject.put("msg", "success");
		jsonObject.put("data", jsonData);

		return jsonObject;


	}

	// 添加具体用户信息
	@PostMapping("/addUser")
	@ResponseBody
	public <T> Object addUser(@RequestBody UserEntity userEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			String a = userService.insertUser(userEntity);
			if (a.equals("success")) {
				object = JSON.toJSON(jsonPackUtils.ok("添加"+userEntity.getHouseNum()+"成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error(a));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("添加"+userEntity.getHouseNum()+"失败"));
			e.printStackTrace();
		}
		return object;
	}

	/**
	 * 修改具体用户xingxi
	 * 修改<2019-12-13> 
	 * 1、修改更新时lastValue为0
	 * 2、修改数据后删除wt_now里的关联数据
	 * 3、修改wt_history表的关联数据
	 * @param <T>
	 * @param userEntity
	 * @return
	 */
	@PostMapping("/updateUser")
	@ResponseBody
	public <T> Object updateUser(@RequestBody UserEntity userEntity) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			int a = userService.updateUser(userEntity);
			if (a != 0) {
				object = JSON.toJSON(jsonPackUtils.ok("更新"+userEntity.getHouseNum()+"成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error("更新"+userEntity.getHouseNum()+"失败"));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("更新"+userEntity.getHouseNum()+"失败"));
			e.printStackTrace();
		}
		return object;
	}

	// 删除具体的用户信息
	@RequestMapping("/deleteUser")
	public <T> Object deleteUser(@RequestParam(value = "id",required = false) Integer id) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		JsonPackUtils<T> jsonPackUtils = new JsonPackUtils<>();
		Object object;
		try {
			int a = userService.deleteUser(id);
			if (a != 0) {
				object = JSON.toJSON(jsonPackUtils.ok("删除成功"));
			} else {
				object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			}
		} catch (Exception e) {
			// TODO: handle exception
			object = JSON.toJSON(jsonPackUtils.error("删除失败"));
			e.printStackTrace();
		}
		return object;
	}

	// 根据条件查询用户信息 update by ljf on 2019-08-09
	@RequestMapping("/queryByOthers")
	public Object queryUsersByO(@RequestParam(value = "page",required = false) Integer page,
			@RequestParam(value = "limit",required = false) Integer limit,
			@RequestParam(value = "houseNum",required = false) String houseNum,
			@RequestParam(value = "deviceNum",required = false) String deviceNum,
			@RequestParam(value = "collectionNum",required = false) String collectionNum,
			@RequestParam(value = "nodeName",required = false) String nodeName,
			@RequestParam(value = "parentId",required = false) Integer parentId) {
		DBIdentifier.setProjectCode(UserConstants.DATA_BASE);
		UserEntity userEntity = new UserEntity();
		if (houseNum != null && houseNum != "") {
			userEntity.setHouseNum(houseNum);
		}
		if (deviceNum != null && deviceNum != "") {
			userEntity.setDeviceNum(deviceNum);
		}
		if (collectionNum != null && collectionNum != "") {
			userEntity.setCollectionNum(collectionNum);
		}
		if (parentId != null) {
			userEntity.setParentId((long)parentId);
		}
		if (nodeName != null & nodeName != "") {
			userEntity.setNodeName(nodeName);
		}
		List<UserEntity> userList = userService.queryUserByOthers(userEntity, page, limit);
		ResponseEntity<UserEntity> responseEntity = new ResponseEntity<UserEntity>();
		System.out.println(JSON.toJSONString(userList,SerializerFeature.PrettyFormat,SerializerFeature.WriteMapNullValue));
		responseEntity.setCode(200);
		responseEntity.setMsg("success");
		responseEntity.setData(userList);
		responseEntity.setCount(userService.queryUsersCount(userEntity, page, limit));
		return JSON.toJSON(responseEntity);
	}

	/**
	 * 设置子元素
	 * 2018.06.09
	 *
	 * @param p
	 * @param permissions
	 */
	private void setChild(UserTreeEntity p, List<UserTreeEntity> permissions, int level) {
		List<UserTreeEntity> child = permissions.parallelStream().filter(a -> a.getParentId().equals(p.getId())).collect(Collectors.toList());
		p.setLevel(level);
		p.setChildren(child);
		if (!CollectionUtils.isEmpty(child)) {
			child.parallelStream().forEach(c -> {
				//递归设置子元素，多级菜单支持
				setChild(c, permissions,level+1);
			});
			p.setLast(false);
		}
	}

}
