package com.mh.eems.mapper;

import java.text.MessageFormat;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.InsertProvider;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.mapping.StatementType;
import org.springframework.stereotype.Component;

import com.mh.eems.entity.DeviceData;
import com.mh.eems.entity.LastDevicesDataEntity;

/**
 * 
 * @ClassName: DeviceDataMapper
 * @Description:TODO(设备数据Mapper)
 * @author: 铭汉科技—LJF
 * @date: 2019年6月13日 上午11:05:17
 *
 */
@Component
@Mapper
@SuppressWarnings("unchecked")
public interface DeviceDataMapper {

	// 执行存储过程，计算返回来的数值
	@Select({"exec pro_photo_data "})
	@Options(statementType=StatementType.CALLABLE)
	void dealDataPro();
	
	
	// 查询是否存在同样的记录值
	@Select("select top 1 count(*) from wt_history where wt_num = #{DeviceID} and cur_date = #{CollectTime}")
	int checkwtHistory(DeviceData deviceData);

	// 查询是否存在同样的记录值
	@Select("select top 1 count(*) from wt_history where wt_num = #{wt_num}")
	int checkwtHistoryTest(@Param("wt_num") String wt_num);

	// 批量插入数据
	@InsertProvider(type = Provider.class, method = "batchInsert")
	int batchInsert(List<DeviceData> deviceDatas);

	class Provider {
		/* 批量插入 */
		public String batchInsert(Map<?, ?> map) {

			List<DeviceData> students = (List<DeviceData>) map.get("list");
			StringBuilder sb = new StringBuilder();
			sb.append("INSERT INTO wt_history (wt_num,cur_flow,cur_date) VALUES ");
			MessageFormat mf = new MessageFormat(
					"(#'{'list[{0}].DeviceID}, #'{'list[{0}].CollectData}, #'{'list[{0}].CollectTime})");

			for (int i = 0; i < students.size(); i++) {
				sb.append(mf.format(new Object[] { i }));
				if (i < students.size() - 1)
					sb.append(",");
			}
			return sb.toString();
		}

		/* 批量删除 */
		public String batchDelete(Map<?, ?> map) {
			List<DeviceData> deviceDatas = (List<DeviceData>) map.get("list");
			StringBuilder sb = new StringBuilder();
			sb.append("DELETE FROM student WHERE id IN (");
			for (int i = 0; i < deviceDatas.size(); i++) {
				sb.append("'").append(deviceDatas.get(i).getDeviceID()).append("'");
				if (i < deviceDatas.size() - 1)
					sb.append(",");
			}
			sb.append(")");
			return sb.toString();
		}
	}

	// 查询采集到的总记录值
	@Select(" select wt_num,cur_flow,MAX(convert(datetime,cur_date,120)) "
			+ "as cur_date from wt_history group by wt_num,cur_flow")
	@Results({ @Result(property = "DeviceID", column = "wt_num"),
			@Result(property = "CollectData", column = "cur_flow"),
			@Result(property = "CollectTime", column = "cur_date") })
	public List<DeviceData> getAllData();
	
	// 查询采集到的最新的记录值
	@Select("select HouseNo,MeterID,DeviceType,Convert(numeric(24,1),ISNULL(LastValue,0.0)) " +
			"as LastValue,LastDate from MeterInstall order by DeviceType ")
	@Results({ @Result(property = "UserNo", column = "HouseNo"),
			@Result(property = "MeterID", column = "MeterID"),
			@Result(property = "HouseNo", column = "DeviceType"),
			@Result(property = "LastValue", column = "LastValue"),
			@Result(property = "LastDate", column = "LastDate"),})
	public List<LastDevicesDataEntity> getLastData();

}
