package com.mh.eems.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.InsertProvider;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectProvider;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.UpdateProvider;
import org.apache.ibatis.mapping.StatementType;

import com.mh.eems.entity.DeviceInstallEntity;
import com.mh.eems.entity.UserEntity;
import com.mh.eems.provider.DeviceInstallProvider;

/**
 * 
 * @ClassName:  DeviceInstallMapper   
 * @Description:TODO(基表安装Mapper类，对数据库操作)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年8月6日 上午10:25:32   
 *
 */
@Mapper
public interface DeviceInstallMapper {
	
	// 通过excel批量导入数据库
	@InsertProvider(type = DeviceInstallProvider.class, method = "batchInsertByExcel")
	int deviceBatchInsert(List<Object> excel2DeviceInstallEntities);
	
	// 查询设备安装信息
	@Results({ 
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "buildingId", column = "building_id"),
		@Result(property = "buildingName", column = "building_name"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "dataCom", column = "data_com"),
		@Result(property = "initialValue", column = "initial_value"),
		@Result(property = "paramName", column = "param_name"),
		@Result(property = "deviceCaliber", column = "device_caliber"),
		@Result(property = "lastValue", column = "last_value"),
		@Result(property = "lastDate", column = "last_date"),
		@Result(property = "communicationType", column = "communication_type"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "deviceType", column = "device_type"),
		@Result(property = "remark", column = "remark"),
		@Result(property = "status", column = "grade")
	})
	@Select("select * from device_install order by id desc offset (#{page} - 1) * #{limit} rows FETCH NEXT #{limit} ROWS ONLY ")
	List<DeviceInstallEntity> queryDevices(@Param("page") int page,@Param("limit") int limit);
	
	// 添加设备安装信息：户号、安装位置、上一级名称ID、上一级名称、表身码、抄表码、初始值、设备ID、设备参数名称、口径、通讯方式、备注、COM口
	@Insert("insert into device_install"
			+ "(house_num,house_address,building_id,building_name,device_num,collection_num,initial_value,param_id,param_name,device_caliber,communication_type,create_time,device_type,remark,data_com) values ("
			+ "#{houseNum},"
			+ "#{houseAddress},"
			+ "(select top 1 id from user_manage where house_num  = #{buildingName}),"
			+ "#{buildingName},"
			+ "#{deviceNum},"
			+ "#{collectionNum},"
			+ "#{initialValue},"
			+ "(select top 1 param_id from device_parameter where param_name = #{paramName}),"
			+ "#{paramName},"
			+ "#{deviceCaliber},"
			+ "#{communicationType},"
			+ "getdate(),"
			+ "#{deviceType},"
			+ "#{remark},"
			+ "#{dataCom}"
			+ ")")
	int insertInstall(DeviceInstallEntity dInstallEntity);
	
	// 修改设备信息：安装位置、上一级名称ID、上一级名称、表身码、抄表码、初始值、设备ID、设备参数名称、口径、通讯方式、备注、基表类型（分表、总表）
	@UpdateProvider(type = DeviceInstallProvider.class, method = "updateInstall")
	int updateInstalls(DeviceInstallEntity dInstallEntity);
	
	//删除被修改表在wt_now表里的记录
	@Delete("delete wt_now where house_num=#{houseNum}")
	void deleteWtNow(String houseNum);

	//判断被修改的表身号对应的户号是否为空
	@Select("select count(*) as houseCount from wt_history where wt_num=#{wt_num} and house_num is null")
	int findHouseNoCount(@Param("wt_num")String wt_num);
	
	//更新抄表标识码对应的户号
	@Update("update wt_history set house_num=#{houseNo}, cur_date = getdate() where wt_num=#{wtNum}")
	void updateHouseNo(@Param("houseNo")String houseNo,@Param("wtNum")String wtNum);
	
	// 根据户号删除设备信息
	@Delete("delete from device_install where house_num = #{houseNum}")
	int deleteInstall(@Param("houseNum") String houseNum);
	
	// 根据条件查询设备信息
	@Results({ 
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "buildingId", column = "building_id"),
		@Result(property = "buildingName", column = "building_name"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "dataCom", column = "data_com"),
		@Result(property = "initialValue", column = "initial_value"),
		@Result(property = "paramName", column = "param_name"),
		@Result(property = "deviceCaliber", column = "device_caliber"),
		@Result(property = "lastValue", column = "last_value"),
		@Result(property = "lastDate", column = "last_date"),
		@Result(property = "communicationType", column = "communication_type"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "deviceType", column = "device_type"),
		@Result(property = "remark", column = "remark"),
		@Result(property = "status", column = "grade")
	})
	@SelectProvider(type = DeviceInstallProvider.class, method = "queryByType")
	List<DeviceInstallEntity> queryType(@Param("dInstallEntity")DeviceInstallEntity dInstallEntity,
			@Param("page") Integer page,@Param("limit") Integer limit,
			@Param("startTime") String startTime, @Param("endTime") String endTime );
	
	// 查询记录总数
	@SelectProvider(type = DeviceInstallProvider.class, method = "queryCount")
	int queryCount(@Param("dInstallEntity")DeviceInstallEntity dInstallEntity, @Param("startTime") String startTime, @Param("endTime") String endTime);

	// 查询是否有相同的户号
	@Select("select top 1 count(*) from device_install where house_num = #{houseNum}")
	int queryDevicesIsSame(DeviceInstallEntity dInstallEntity);
	
	// 根据户号查询对应的基表信息
	@SelectProvider(type = DeviceInstallProvider.class, method = "queryDeviceByHouseNum")
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "nodeName", column = "node_name"),
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "parentId", column = "parent_id")
	})
	List<UserEntity> queryDeviceByHouseNum(@Param("houseNum") String houseNum);
	
	// 执行存储过程，更新水表状态和网关状态
	@Select("exec pro_update_status ")
	@Options(statementType=StatementType.CALLABLE)
	void updateDeviceGateway();
}
