package com.mh.eems.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.InsertProvider;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectProvider;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.UpdateProvider;

import com.mh.eems.entity.ExchangeDeviceEntity;
import com.mh.eems.provider.DeviceInstallProvider;
import com.mh.eems.provider.ExchangeDeviceProvider;

/**
 * 
 * @ClassName:  ExchangeDeviceMapper   
 * @Description:TODO(表号更换Mapper类，对数据库操作)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年11月18日 上午11:12:39   
 *
 */
@Mapper
public interface ExchangeDeviceMapper {
	
	// 根据条件查询出记录
	// 根据条件查询设备信息
	@Results({ 
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "buildingName", column = "building_name"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "dataCom", column = "data_com"),
		@Result(property = "lastValue", column = "last_value"),
		@Result(property = "paramType", column = "param_type"),
		@Result(property = "newDeviceNum", column = "new_device_num"),
		@Result(property = "newCollectionNum", column = "new_collection_num"),
		@Result(property = "newValue", column = "new_value"),
		@Result(property = "updateTime", column = "update_time"),
		@Result(property = "remark", column = "remark")
	})
	@SelectProvider(type = ExchangeDeviceProvider.class, method = "queryByType")
	List<ExchangeDeviceEntity> queryType(@Param("exchangeDeviceEntity")ExchangeDeviceEntity exchangeDeviceEntity,
			@Param("page") Integer page,@Param("limit") Integer limit);
	
	// 查询记录总数
	@SelectProvider(type = ExchangeDeviceProvider.class, method = "queryCount")
	int queryCount(@Param("exchangeDeviceEntity")ExchangeDeviceEntity exchangeDeviceEntity,
			@Param("page") Integer page,@Param("limit") Integer limit);
	
	
	// 插入记录
	@Insert("insert into exchange_devices(house_num,house_address,building_name,"
			+ "device_num,collection_num,data_com,last_value,param_type,"
			+ "new_device_num,new_collection_num,new_value,update_time,remark) "
			+ "values (#{houseNum},#{houseAddress},#{buildingName},#{deviceNum},#{collectionNum},#{dataCom},#{lastValue},"
			+ "#{paramType},#{newDeviceNum},#{newCollectionNum},#{newValue},#{updateTime},#{remark})")
	void insertExchangeDevice(ExchangeDeviceEntity exchangeDeviceEntity);
	
	// 更新记录值
	@UpdateProvider(type = ExchangeDeviceProvider.class, method = "updateDevice")
	void updateExchangeDevice(ExchangeDeviceEntity exchangeDeviceEntity);
	
	//判断被修改的表身号对应的户号是否为空
	@Select("select count(*) as houseCount from wt_history where wt_num=#{wt_num} and house_num is null")
	int findHouseNoCount(@Param("wt_num")String wt_num);
	
	//更新抄表标识码对应的户号
	@Update("update wt_history set house_num=#{houseNo},cur_date=getdate() where wt_num=#{wtNum}")
	void updateHouseNo(@Param("houseNo")String houseNo,@Param("wtNum")String wtNum);
	
	//删除被修改表在wt_now表里的记录
	@Delete("delete wt_now where house_num=#{houseNum}")
	void deleteWtNow(@Param("houseNum")String houseNum);
	
	// 根据ID删除记录值
	@Delete("delete from exchange_devices where id = #{id}")
	void deleteExchangeDevice(@Param("id")Long id);
	
	// 批量插入数据库
	// 通过excel批量导入数据库
	@InsertProvider(type = ExchangeDeviceProvider.class, method = "batchInsertDeviceByExcel")
	int batchInsertDevice(List<Object> excel2ExchangeDeviceEntities);


}
