package com.mh.eems.mapper;

import java.util.List;

import com.mh.eems.provider.DeviceInstallProvider;
import org.apache.ibatis.annotations.*;

import com.mh.eems.dto.GatewayStatusDto;
import com.mh.eems.entity.GateWayManageEntity;
import com.mh.eems.provider.GatewayManageProvider;

/**
 * 
 * @ClassName:  GatewayManageMapper   
 * @Description:TODO(网关设备管理Mapper)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年7月31日 下午3:31:23   
 *
 */
@Mapper
public interface GatewayManageMapper {
	
//	Long gatewayId;				// 网关ID 
//	String gatewayName;			// 网关名称
//	String gatewayIp;			// 网关IP
//	String gatewayAddress;		// 网关安装位置
//	String dataCom;				// COM口标志
//	int loop;					// 采集周期时间
//	Date connectTime;			// 最近连接时间

	// 通过excel批量导入数据库
	@InsertProvider(type = GatewayManageProvider.class, method = "batchInsertByExcel")
	int gatewayBatchInsert(List<Object> excel2gatewayInstallEntities);
	// 按条件查询
	@SelectProvider(type = GatewayManageProvider.class, method = "queryGatewayByOthers")
	@Results({ @Result(property = "gatewayId", column = "gateway_id"),
		@Result(property = "gatewayName", column = "gateway_name"),
		@Result(property = "gatewayIp", column = "gateway_ip"),
		@Result(property = "gatewayAddress", column = "gateway_address"),
		@Result(property = "dataCom", column = "data_com"),
		@Result(property = "loop", column = "collection_loop"),
		@Result(property = "connectTime", column = "connect_date"),
		@Result(property = "status", column = "grade"),
		@Result(property = "createDate", column = "create_date"),
		@Result(property = "internetCard", column = "internet_card"),
		@Result(property = "operator", column = "operator")
	})
	List<GateWayManageEntity> queryGatewayByOther(@Param("gm") GateWayManageEntity gm, @Param("page") Integer page,
			@Param("limit") Integer limit,@Param("start")String start,@Param("end")String end);
	
	// 查询记录数
	@SelectProvider(type = GatewayManageProvider.class, method = "queryGatewayCount")
	int queryGatewayCount(@Param("gm") GateWayManageEntity gm, @Param("page") Integer page,
			@Param("limit") Integer limit, @Param("start")String start,@Param("end")String end);
	
	// 添加记录,并返回id值
	@Options(useGeneratedKeys = true, keyProperty = "id", keyColumn = "id")
	@Insert("insert into gateway_manage(gateway_id,gateway_name,gateway_ip,gateway_address,data_com,collection_loop,internet_card,operator,create_date) values "
			+ "(#{gatewayId},"
			+ "#{gatewayName},"
			+ "#{gatewayIp},"
			+ "#{gatewayAddress},"
			+ "#{dataCom},"
			+ "#{loop},"
			+ "#{internetCard},"
			+ "#{operator},"
			+ "getdate())")
	int addGateway(GateWayManageEntity gateWayManageEntity);
	
	// 查询是否有重复gatewayId和COM口配置
	@SelectProvider(type = GatewayManageProvider.class, method = "queryIsSame")
	@Results({ @Result(property = "gatewayId", column = "gateway_id"),
		@Result(property = "gatewayName", column = "gateway_name"),
		@Result(property = "gatewayIp", column = "gateway_ip"),
		@Result(property = "gatewayAddress", column = "gateway_address"),
		@Result(property = "dataCom", column = "data_com"),
		@Result(property = "loop", column = "collection_loop"),
		@Result(property = "connectTime", column = "connect_date"),
		@Result(property = "status", column = "grade")
	})
	GateWayManageEntity queryIsSame(@Param("gm") GateWayManageEntity gm);
	
	// 修改网关信息
	@UpdateProvider(type = GatewayManageProvider.class, method = "updateGateway")
	int updateGateway(@Param("gm")GateWayManageEntity gm);
	
	// 通过gatewayId删除
	@Delete("delete from gateway_manage where gateway_id = #{gatewayId}")
	int deleteGaway(@Param("gatewayId") long gatewayId);

	// 查询网关状态值（使用在饼图）
	@Select("select '离线' as status,count(*) as total_num from gateway_manage where grade = 0 union all " + 
			"select '正常' as status,count(*) as total_num from gateway_manage where grade = 1 union all " + 
			"select '异常' as status,count(*) as total_num from gateway_manage where grade = 2")
	@Results({ @Result(property = "status", column = "status"),
		@Result(property = "totalNum", column = "total_num")
	})
	List<GatewayStatusDto> queryStatusForGateway();
}
