package com.mh.eems.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.type.JdbcType;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import com.mh.eems.entity.PermissionEntity;

@Component
@Mapper
@Primary
public interface PermissionMapper {

	@Select("select * from sys_permission t order by t.sort")
	List<PermissionEntity> listAll();

	@Select("select * from sys_permission t where t.type = 1 order by t.sort")
	List<PermissionEntity> listParents();

	@Select("select distinct p.* from sys_permission p "
			+ " inner join sys_role_permission rp on p.id = rp.permission_id "
			+ " inner join sys_role_user ru on ru.role_id = rp.role_id "
			+ " where ru.user_id = #{userId} and rp.is_status = 1 and ru.role_id = #{roleId} order by p.sort")
	@Results({
	    @Result(column="id", property="id", jdbcType=JdbcType.INTEGER, id=true),
	    @Result(column="parent_id", property="parentId", jdbcType=JdbcType.INTEGER),
	    @Result(column="name", property="name", jdbcType=JdbcType.VARCHAR),
//	    @Result(column="css", property="css", jdbcType=JdbcType.VARCHAR),
	    @Result(column="css", property="titleIcon", jdbcType=JdbcType.VARCHAR),
	    @Result(column="href", property="href", jdbcType=JdbcType.VARCHAR),
	    @Result(column="type", property="type", jdbcType=JdbcType.INTEGER),
	    @Result(column="permission", property="permission", jdbcType=JdbcType.VARCHAR),
	    @Result(column="sort", property="sort", jdbcType=JdbcType.INTEGER)
	})
	List<PermissionEntity> listByUserId(@Param("userId")Long userId,@Param("roleId")int roleId);

	@Select("select p.*,rp.is_status from sys_permission p "
			+ " inner join sys_role_permission rp on p.id = rp.permission_id where rp.role_id = #{roleId} order by p.sort")
	@Results({
	    @Result(column="id", property="id", jdbcType=JdbcType.INTEGER, id=true),
	    @Result(column="parent_id", property="parentId", jdbcType=JdbcType.INTEGER),
	    @Result(column="name", property="name", jdbcType=JdbcType.VARCHAR),
	    @Result(column="css", property="titleIcon", jdbcType=JdbcType.VARCHAR),
	    @Result(column="href", property="href", jdbcType=JdbcType.VARCHAR),
	    @Result(column="type", property="type", jdbcType=JdbcType.INTEGER),
	    @Result(column="permission", property="permission", jdbcType=JdbcType.VARCHAR),
	    @Result(column="sort", property="sort", jdbcType=JdbcType.INTEGER),
	    @Result(column="is_status", property="isStatus", jdbcType=JdbcType.INTEGER)
	})
	List<PermissionEntity> listByRoleId(Long roleId);

	@Select("select * from sys_permission t where t.id = #{id}")
	PermissionEntity getById(Long id);

	@Insert("insert into sys_permission(parentId, name, css, href, type, permission, sort) values(#{parentId}, #{name}, #{css}, #{href}, #{type}, #{permission}, #{sort})")
	int save(PermissionEntity permission);

	@Update("update sys_permission t set parentId = #{parentId}, name = #{name}, css = #{css}, href = #{href}, type = #{type}, permission = #{permission}, sort = #{sort} where t.id = #{id}")
	int update(PermissionEntity permission);

	@Delete("delete from sys_permission where id = #{id}")
	int delete(Long id);
	
	@Delete("delete from sys_permission where parentId = #{id}")
	int deleteByParentId(Long id);

	@Delete("delete from sys_role_permission where permissionId = #{permissionId}")
	int deleteRolePermission(Long permissionId);
}
