package com.mh.eems.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Many;
import org.springframework.stereotype.Component;

import com.mh.eems.dto.RoleAndPermissionDto;
import com.mh.eems.entity.PermissionEntity;
import com.mh.eems.entity.RoleEntity;

/**
 * 
 * @ClassName:  RoleMapper   
 * @Description:TODO(角色管理：查询数据库内容)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年7月24日 上午11:23:05   
 *
 */
@Component
@Mapper
public interface RoleMapper {
	
	// 查询全部
	@Select("select * from sys_role order by id asc offset (#{page} - 1) * #{limit} rows FETCH NEXT #{limit} ROWS ONLY")
	@Results({ @Result(property = "roleId", column = "id"),
		@Result(property = "roleName", column = "name"),
		@Result(property = "desc", column = "description"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "updateTime", column = "update_time")
	})
	List<RoleEntity> queryAllRoles(@Param("page") int page,@Param("limit") int limit);
	@Select("select * from sys_role order by id asc")
	@Results({ @Result(property = "roleId", column = "id"),
		@Result(property = "roleName", column = "name"),
		@Result(property = "desc", column = "description"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "updateTime", column = "update_time")
	})
	List<RoleEntity> queryAllRoles1();
	
	// 根据用户名查询
	@Select("select top 1 * from sys_role where name = #{roleName}")
	@Results({ @Result(property = "roleId", column = "id"),
		@Result(property = "roleName", column = "name"),
		@Result(property = "desc", column = "description"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "updateTime", column = "update_time")
	})
	RoleEntity queryRoleByRoleName(RoleEntity roleEntity);
	
	// 查询记录数
	@Select("select top 1 count(*) from sys_role")
	int queryRoleCount();
	
	// 根据ID查询角色
	@Select("select * from sys_role where id = #{id}")
	@Results({ @Result(property = "roleId", column = "id"),
		@Result(property = "roleName", column = "name"),
		@Result(property = "desc", column = "description"),
		@Result(property = "isStatus", column = "is_status")
	})
	List<RoleEntity> queryRoleByID(@Param("id")Long id);
	
	// 根据用户ID查询角色
	@Select("select r.*,ru.is_status from sys_role r "
			+ " inner join sys_role_user ru on r.id = ru.role_id where ru.user_id = #{userId} order by ru.is_status desc ")
	@Results({ @Result(property = "roleId", column = "id"),
		@Result(property = "roleName", column = "name"),
		@Result(property = "desc", column = "description"),
		@Result(property = "isStatus", column = "is_status")
	})
	List<RoleEntity> queryRoleByUserID(@Param("user_id")Long userId);
	
	// 根据roleId查询出对应的用户和权限名称
	@Select("select top 1 * from sys_role_permission where role_id = #{id}")
	@Results({ 
		@Result(property = "roles", column = "role_id",
            many = @Many(select = "com.mh.eems.mapper.RoleMapper.queryRoleByID")),
		@Result(property = "permissions", column = "role_id",
        many = @Many(select = "com.mh.eems.mapper.PermissionMapper.listByRoleId"))
		
	})
	List<RoleAndPermissionDto> queryRoleAndPermission(@Param("id")Long id);
	
	// 增加角色
	@Insert("insert into sys_role(name,description,create_time,update_time)"
			+ " values(#{roleName},#{desc},getdate(),getdate())")
	@Options(useGeneratedKeys = true, keyProperty = "roleId", keyColumn = "id")		// 获取先添加的id值
	int insertRole(RoleEntity roleEntity); 
	
	// 往role和permission中间表sys_role_permission插入全部数据
	@Insert("insert into sys_role_permission(permission_id,role_id,is_status) " + 
			" select a.id,b.id,0 from (select id from sys_permission) as a left join " + 
			" (select id from sys_role where id not in(select role_id from sys_role_permission)) as b on 1=1 ")
	int insertRoleAndPermission();
	
	// 往role和user中间表sys_role_user插入数据
	@Insert("insert into sys_role_user(user_id,role_id,is_status) " + 
			" select a.id,b.id,0 from (select id from sys_user) as a left join "
			+ " (select id from sys_role where id not in(select role_id from sys_role_user)) as b on 1=1")
	int InsertRoleAndUser();
	
	// 根据ID删除角色
	@Delete("delete sys_role where id = #{roleId}")
	int deleteRoleById(@Param("roleId")Object roleId);
	
	// 删除对应的sys_role_permission表中对应的权限值
	@Delete("delete sys_role_permission where role_id = #{roleId}")
	int deleteRolePermissionById(@Param("roleId")Object roleId);
	
	// 删除对应的sys_role_user中含有的role值
	@Delete("delete sys_role_user where role_id = #{roleId}")
	int deleteRoleAndUserById(@Param("roleId")Object roleId);
	
	// 更新sys_role值
	@Update("update sys_role set name = #{roleName},description = #{desc},update_time = getdate() where id = #{roleId}")
	int updateRoleById(RoleEntity roleEntity);
	
	// 更新sys_role_permission
	@Update({"<script>" +
            "<foreach collection=\"pList\" item=\"item\" separator=\";\">" +
            " UPDATE " +
            " sys_role_permission " +
            " SET is_status = #{item.isStatus} " +
            "  WHERE " +
            "   role_id = #{item.sort,jdbcType=INTEGER} " +
            "   AND permission_id = #{item.id} " +
            "</foreach>" +
            "</script>"})
    int updateRolePermission(@Param("pList") List<PermissionEntity> pList);

}
