package com.mh.eems.mapper;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Many;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectProvider;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.UpdateProvider;
import org.apache.ibatis.mapping.StatementType;
import org.springframework.stereotype.Component;

import com.mh.eems.dto.UserAndRoleDto;
import com.mh.eems.dto.UserDto;
import com.mh.eems.entity.RoleEntity;
import com.mh.eems.entity.SysUserEntity;
import com.mh.eems.provider.SysUserProvider;


/**
 * 
 * @ClassName:  SysUserMapper   
 * @Description:TODO(用户Mapper)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年6月14日 下午4:24:48   
 *
 */
//@Component
@Mapper
public interface SysUserMapper {

	// 删除系统用户和删除sys_role_user的关联值 update by ljf on 2019-07-27
	@Delete("delete sys_user where id = #{userId}")
	int deleteUser(@Param("userId") Long userId);
	@Delete("delete from sys_role_user where user_id = #{userId}")
	int deleteUserRole(@Param("userId")Long userId);
	
	//更新用户状态  update by ljf on 2019-07-27
	@Update("update sys_user set status = #{status},update_time = getdate() where id = #{userId}")
	int updateStatus(@Param("userId") int userId, @Param("status") int status);
	
	//往sys_role_user插入全部角色
	@Insert("insert into sys_role_user(user_id,role_id,is_status) " + 
			" select b.id as user_id,a.id as role_id,0 as is_status from (select id from sys_role) as a left join "
			+ " (select id from sys_user where id not in(select user_id from sys_role_user)) as b on 1=1")
	int insertUserAndRole();
	
	// 更新sys_role_user的对应的user_id
	@Update("update sys_role_user set is_status = 1 where user_id = #{userId} and role_id = #{roleId}")
	int updateUserById(@Param("userId") Long userId,@Param("roleId") Long roleId); 

	// 更新sys_role_user的对应的user_id
	@Update("update sys_role_user set is_status = 0 where user_id = #{userId}")
	int updateRoleUserById(@Param("userId") Long userId); 
	
	// 更新sys_role_user的对应的user_id
	@Update("update sys_role_user set is_status = 1 where user_id = #{userId} and role_id = #{roleId}")
	int updateUserAndRoleById(@Param("userId") Long userId, @Param("roleId") int roleId); 
	
	@Options(useGeneratedKeys = true, keyProperty = "id", keyColumn = "id")
	@Insert("insert into sys_user(username, password, salt, nickname, status, create_time, update_time) "
			+ " values(#{userName}, #{password}, #{salt}, #{nickName}, #{status}, GETDATE(), GETDATE())")
	int save(UserDto userDto);

	@Select("select * from sys_user t where t.id = #{id}")
	SysUserEntity getById(Long id);
	
	@Select("select * from sys_user t where t.id = #{id}")
	List<SysUserEntity> getByUserId(Long id);

//	@Select("select top 1 RoleID,OpName,Password,salt,status from SysOpt t where t.OpName = #{username}")
	@Select("select top 1 id,username,password,salt,status from sys_user t where t.username = #{username}")
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "userName", column = "username"),
		@Result(property = "password", column = "password"),
		@Result(property = "status", column = "status"),
		@Result(property = "salt", column = "salt")})
	SysUserEntity getUser(String username);
	
	@Select({"exec pro_photo_data_test "})
	@Options(statementType=StatementType.CALLABLE)
//	@Select("select top 1 RoleID,OpName,Password,salt,status from SysOpt t where t.OpName = #{username}")
	@Results({ @Result(property = "id", column = "RoleID"),
		@Result(property = "username", column = "OpName"),
		@Result(property = "password", column = "Password"),
		@Result(property = "status", column = "status"),
		@Result(property = "salt", column = "salt")})
	List<SysUserEntity> getUserTest();

	@Update("update sys_user set password = #{password},update_time = getdate() where id = #{id}")
	int changePassword(@Param("id") Long id, @Param("password") String password);

	Integer count(@Param("params") Map<String, Object> params);

	List<SysUserEntity> list(@Param("params") Map<String, Object> params, @Param("offset") Integer offset,
			@Param("limit") Integer limit);

	int saveUserRoles(@Param("userId") Long userId, @Param("roleIds") List<Long> roleIds);

	@UpdateProvider(type = SysUserProvider.class, method = "updateUser")
	int update(UserDto userDto);
	
	// 查询全部的系统用户 update by ljf on 2019-07-26
//	@Select("select distinct u.id,u.username,u.nickname,u.create_time,u.status from sys_user u  " + 
//			" inner join sys_role_user ru on u.id = ru.user_id " + 
//			" inner join sys_role r on ru.role_id = r.id order by u.id")
	@Select("select distinct u.id,u.username,u.nickname,u.create_time,u.update_time,u.status from sys_user u "
			+ " order by id asc offset (#{page} - 1) * #{limit} rows FETCH NEXT #{limit} ROWS ONLY ")
	@Results({
		@Result(property = "id", column = "id"),
		@Result(property = "userName", column = "username"),
		@Result(property = "nickName", column = "nickname"),
		@Result(property = "createTime", column = "create_time"),
		@Result(property = "updateTime", column = "update_time"),
		@Result(property = "status", column = "status")
	})
	List<SysUserEntity> queryAllUsers(@Param("page")int page, @Param("limit") int limit);
	
	// 查询系统用户数
	@Select("select top 1 count(*) from sys_user")
	int queryUserCount();
	
	// 根据ID查询系统用户和所属于的角色
	@Select("select top 1 user_id,role_id from sys_role_user where user_id = #{user_id}")
	@Results({ 
		@Result(property = "roles", column = "user_id",
            many = @Many(select = "com.mh.eems.mapper.RoleMapper.queryRoleByUserID")),
		@Result(property = "users", column = "user_id",
        many = @Many(select = "com.mh.eems.mapper.SysUserMapper.getByUserId"))
		
	})
	List<UserAndRoleDto> querySysUserAndRoleByUserId(@Param("user_id") Long userId);
	
	// 根据ID查询系统用户和所属于的角色
	@Select("select top 1 role_id from sys_role_user where user_id = #{user_id} and is_status = 1")
	int querySysUserAndRoleByUserId1(@Param("user_id") Long userId);
}
