package com.mh.eems.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.InsertProvider;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.SelectProvider;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.annotations.UpdateProvider;

import com.mh.eems.entity.DeviceData;
import com.mh.eems.entity.Excel2UserEntity;
import com.mh.eems.entity.UserEntity;
import com.mh.eems.entity.UserTreeEntity;
import com.mh.eems.mapper.DeviceDataMapper.Provider;
import com.mh.eems.provider.UserProvider;

/**
 * 
 * @ClassName:  UserMapper   
 * @Description:TODO(用户资料管理数据库查询接口)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年8月1日 下午3:04:33   
 *
 */
@Mapper
public interface UserMapper {
	
	// 通过excel批量导入数据库
	@InsertProvider(type = UserProvider.class, method = "batchInsertByExcel")
	int batchInsert(List<Object> excel2UserEntities);
	
	// 查询树形结构
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "title", column = "house_num"),
		@Result(property = "parentId", column = "parent_id"),
		@Result(property = "level", column = "node_type")
	})
	@Select("select * from user_manage where node_type != 99")
	List<UserTreeEntity> queryUserTree();
	
	// 添加树形结构节点,并返回ID值
	@Options(useGeneratedKeys = true, keyProperty = "id", keyColumn = "id")
	@Insert("insert into user_manage(house_num,parent_id,node_type,update_time)values("
			+ "#{title},"
			+ "#{parentId},"
			+ "#{level},"
			+ "getdate()"
			+ ")")
	int insertUserTree(UserTreeEntity userTreeEntity);
	
	// 更新树形结构节点
	@Update("update user_manage set house_num = #{title},update_time = getdate() where id = #{id}")
	int updateUserTree(UserTreeEntity userTreeEntity);
	
	// 删除树形结构中的节点
	@Insert("delete from user_manage where id = #{id}")
	int deleteUserTree(@Param("id")int id);
	
	// 根据ID查询树形结构值
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "title", column = "house_num"),
		@Result(property = "parentId", column = "parent_id"),
		@Result(property = "level", column = "node_type")
	})
	@Select("select * from user_manage where id = #{id}")
	UserTreeEntity queryUserTreeById(@Param("id") int id);
	
	// 根据父id查询对应的对应节点的基表安装信息值
	@Select("select *,#{userTreeEntity.title} as nodeName from user_manage where parent_id = #{userTreeEntity.id} and node_type = 99"
			+ " order by id asc offset (#{page} - 1) * #{limit} rows FETCH NEXT #{limit} ROWS ONLY ")
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "nodeType", column = "node_type"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "remark", column = "remark"),
		@Result(property = "updateTime", column = "update_time")
	})
	List<UserEntity> queryUsers(UserTreeEntity userTreeEntity,@Param("page") Integer page,@Param("limit") Integer limit);
	
	// 根据ID查询节点信息
	@Select("select *,#{title} as nodeName from user_manage where id = #{id}")
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "remark", column = "remark"),
		@Result(property = "updateTime", column = "update_time")
	})
	List<UserEntity> queryUserById(@Param("title")String title,@Param("id") int id);
	
	// 查询记录总数
	@SelectProvider(type=UserProvider.class, method = "queryCountByOther")
	int queryUsersCount(@Param("user") UserEntity userEntity,@Param("page") Integer page,@Param("limit") Integer limit);
	
	
	// 根据ID删除记录
	@Delete("delete from user_manage where id = #{id}")
	int deleteUser(@Param("id") int id);
	
	// 添加用户信息,并返回ID值,为了区分树形结构，添加最后节点用户信息的节点类型是99
	@Options(useGeneratedKeys = true, keyProperty = "id", keyColumn = "id")
	@Insert("insert into user_manage(house_num,node_type,house_address,device_num,collection_num,remark,parent_id,update_time) values ("
			+ "#{houseNum},"
			+ "99,"
			+ "#{houseAddress},"
			+ "#{deviceNum},"
			+ "#{collectionNum},"
			+ "#{remark},"
			+ "#{parentId},"
			+ "getdate()"
			+ ")")
	int insertUser(UserEntity userEntity);
	
	// 更新用户信息
	@UpdateProvider(type = UserProvider.class, method = "updateUser")
	int updateUser(@Param("userEntity")UserEntity userEntity);
	
	// 查询是否有相同的户号
	@Select("select top 1 count(*) from user_manage where house_num = #{houseNum} and node_type = 99")
	int queryUsersIsSame(UserEntity userEntity);
	
	
	// 更具条件查询信息
	@SelectProvider(type=UserProvider.class, method = "queryUsersByOthered")
	@Results({ @Result(property = "id", column = "id"),
		@Result(property = "nodeName", column = "node_name"),
		@Result(property = "houseNum", column = "house_num"),
		@Result(property = "houseAddress", column = "house_address"),
		@Result(property = "deviceNum", column = "device_num"),
		@Result(property = "collectionNum", column = "collection_num"),
		@Result(property = "remark", column = "remark"),
		@Result(property = "updateTime", column = "update_time")
	})
	List<UserEntity> queryUserByOthers(@Param("user") UserEntity userEntity,@Param("page") Integer page,@Param("limit") Integer limit);

}
