package com.mh.eems.service.impl;

import java.util.List;
import java.util.UUID;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.shiro.crypto.hash.SimpleHash;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.mh.eems.constants.UserConstants;
import com.mh.eems.dto.UserAndRoleDto;
import com.mh.eems.dto.UserDto;
import com.mh.eems.entity.RoleEntity;
import com.mh.eems.entity.SysUserEntity;
import com.mh.eems.mapper.RoleMapper;
import com.mh.eems.mapper.SysUserMapper;
import com.mh.eems.service.SysUserService;
import com.mh.eems.util.SysLogUtil;
import com.mh.eems.util.SysUserUtil;

/**
 * 
 * @ClassName:  UserServiceImpl   
 * @Description:TODO(用户服务实现类)   
 * @author: 铭汉科技—LJF 
 * @date:   2019年6月14日 下午4:28:45   
 *
 */
@Primary
@Service
//@Transactional(transactionManager="sqlserverTransactionManager")
public class SysUserServiceImpl implements SysUserService {

	private static final Logger log = LoggerFactory.getLogger("adminLogger");
	
	@Autowired
	private SysUserMapper userMapper;
	
	@Autowired
	private RoleMapper roleMapper;
	
//	SysLogUtil sysLogsUtil = new SysLogUtil();
	
	@Override
	@Transactional
	public UserDto saveUser(UserDto userDto) {
		System.out.println(userDto.toString());
		userDto.setSalt(DigestUtils
				.md5Hex(UUID.randomUUID().toString() + System.currentTimeMillis() + UUID.randomUUID().toString()));
		userDto.setPassword(passwordEncoder(userDto.getPassword(), userDto.getSalt()));
		userMapper.save(userDto);
		// 保存用户的所属角色
		userMapper.insertUserAndRole();
		
		System.out.println("saveUser: "+userDto.getId()+","+userDto.getRoleName());
		saveUserRoles(userDto.getId(),userDto.getRoleName());

		log.debug("新增用户{}", userDto.getUserName());
		
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("系统用户管理", "保存用户："+userDto.getUserName(), "添加");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userDto;
	}
	private void saveUserRoles(Long userId,String roleName) {
		if (userId != null) {
			// 查询角色名称的ID
			RoleEntity roleEntity = new RoleEntity();
			roleEntity.setRoleName(roleName);
			RoleEntity roleEntity2 = roleMapper.queryRoleByRoleName(roleEntity);
			System.out.println("saveUserRoles:"+roleEntity2.toString());
			// 更新对应的角色
			userMapper.updateUserById(userId, roleEntity2.getRoleId());
		}
	}

	@Override
	public SysUserEntity updateUser(UserDto userDto) {
		// 查询出用户ID
//		User user = userMapper.getUser(userDto.getUserName());
//		userDto.setId((long)user.getId());
		// 如果password=“”时，说明没有更新,不用更新密码
		if (userDto.getPassword() == null) {
			int a =	userMapper.update(userDto);
//			if (a != 0) {
//				throw new IllegalArgumentException("不能相同用户名，请重新输入");
//			}	
		} else {
			int b = userMapper.update(userDto);
//			if (b != 0) {
//				throw new IllegalArgumentException("不能相同用户名，请重新输入");
//			}
			// 更新密码
			changePassword(userDto.getUserName(), userDto.getPassword());
		}
		// 查询角色名称的ID
		RoleEntity roleEntity = new RoleEntity();
		roleEntity.setRoleName(userDto.getRoleName());
		RoleEntity roleId = roleMapper.queryRoleByRoleName(roleEntity);
		// 更新对应的角色
		// 先全部设置系统用户拥有的角色全部置为0
		userMapper.updateRoleUserById(userDto.getId());
		userMapper.updateUserById(userDto.getId(), roleId.getRoleId());
		// 更新会话
//		updateUserSession(userDto.getId());
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户", 
					"更新用户："+userDto.getPassword() == ""?userDto.getUserName():userDto.getUserName()+"密码",
					"更新");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userDto;
	}

	private void updateUserSession(Long id) {
		SysUserEntity current = SysUserUtil.getCurrentUser();
		if (current.getId().equals(id)) {
			SysUserEntity user = userMapper.getById(id);
			SysUserUtil.setUserSession(user);
		}
	}

	@Override
	public String passwordEncoder(String credentials, String salt) {
		// TODO Auto-generated method stub
		Object object = new SimpleHash("MD5", credentials, salt, UserConstants.HASH_ITERATIONS);
		return object.toString();
	}

	@Override
	public SysUserEntity getUser(String username) {
		// TODO Auto-generated method stub
		return userMapper.getUser(username);
	}

	@Override
	public void changePassword(String username, String newPassword) {
		// TODO Auto-generated method stub
		SysUserEntity u = userMapper.getUser(username);
		if (u == null) {
			throw new IllegalArgumentException("用户不存在");
		}

//		if (!u.getPassword().equals(passwordEncoder(oldPassword, u.getSalt()))) {
//			throw new IllegalArgumentException("密码错误");
//		}

		userMapper.changePassword(u.getId(), passwordEncoder(newPassword, u.getSalt()));

		log.debug("修改{}的密码", username);
	}

	@Override
	public List<SysUserEntity> getUserTest() {
		// TODO Auto-generated method stub
		return userMapper.getUserTest();
	}

	@Override
	public List<SysUserEntity> queryAllUsers(int page,int limit) {
		// TODO Auto-generated method stub
		return userMapper.queryAllUsers(page,limit);
	}

	@Override
	public int queryUserCount() {
		// TODO Auto-generated method stub
		return userMapper.queryUserCount();
	}

	@Override
	public List<UserAndRoleDto> querySysUserAndRoleByUserId(int userId) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("系统用户管理", 
					"根据用户ID"+userId+"查询用户以及拥有角色类型",
					"查询");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.querySysUserAndRoleByUserId((long)userId);
	}

	@Override
	public int updateUserById(int user_id, int role_id) {
		// TODO Auto-generated method stub
		return 0;
	}
	@Override
	public int updateStatus(int userId, int status) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("系统用户管理", 
					"通过ID："+userId+"更新用户",
					"更新");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.updateStatus(userId, status);
	}
	@Override
	public int deleteUser(Long userId) {
		// TODO Auto-generated method stub
		// 先删除系统用户
		int a = userMapper.deleteUser(userId);
		if (a != 0) {
			// 插入日志
			try {
				SysLogUtil sysLogsUtil = new SysLogUtil();
				sysLogsUtil.logSave("系统用户管理", 
						"通过ID删除用户",
						"删除");
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			userMapper.deleteUserRole(userId);
		}
		return a;
	}
}
