package com.mh.eems.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import com.mh.eems.annotation.LogAnnotation;
import com.mh.eems.entity.DeviceInstallEntity;
import com.mh.eems.entity.Excel2UserEntity;
import com.mh.eems.entity.UserEntity;
import com.mh.eems.entity.UserTreeEntity;
import com.mh.eems.mapper.DeviceInstallMapper;
import com.mh.eems.mapper.UserMapper;
import com.mh.eems.service.UserService;
import com.mh.eems.util.SysLogUtil;

@Primary
@Service
public class UserServiceImpl implements UserService {

	@Autowired
	private UserMapper userMapper;
	
	@Autowired
	private DeviceInstallMapper deviceInstallMapper;
	
	@LogAnnotation(type = "查询", logs = "查询树形结构内容", module = "用户资料管理")
	@Override
	public List<UserTreeEntity> queryUserTree() {
		// TODO Auto-generated method stub
		return userMapper.queryUserTree();
	}

	@Override
	public int insertUserTree(UserTreeEntity userTreeEntity) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户资料管理", 
					"插入用户信息信息："+userTreeEntity.toString(),
					"添加");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.insertUserTree(userTreeEntity);
	}

	@Override
	public int updateUserTree(UserTreeEntity userTreeEntity) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户资料管理", 
					"更新用户信息："+userTreeEntity.toString(),
					"更新");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.updateUserTree(userTreeEntity);
	}

	@Override
	public int deleteUserTree(int id) {
		// TODO Auto-generated method stub
		// 查询树形结构
		UserTreeEntity userTreeEntity = userMapper.queryUserTreeById(id);
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户资料管理", 
					"删除用户信息："+userTreeEntity.toString(),
					"删除");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.deleteUserTree(id);
	}

	@Override
	public List<UserEntity> queryUsers(UserTreeEntity userTreeEntity,Integer page,Integer limit) {
		// TODO Auto-generated method stub
		return userMapper.queryUsers(userTreeEntity,page,limit);
	}

	@Override
	public int deleteUser(int id) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户资料管理", 
					"删除用户信息："+id,
					"删除");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return userMapper.deleteUser(id);
	}

	@Override
	public String insertUser(UserEntity userEntity) {
		// TODO Auto-generated method stub
		int a = userMapper.queryUsersIsSame(userEntity);
		if (a == 0) {
			userMapper.insertUser(userEntity);
			// 插入日志
			try {
				SysLogUtil sysLogsUtil = new SysLogUtil();
				sysLogsUtil.logSave("用户资料管理", 
						"添加用户信息："+userEntity.toString(),
						"添加");
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return "success";
		} else {
			return "户号不能重复添加";
		}
		 
	}

	@Override
	public int updateUser(UserEntity userEntity) {
		// TODO Auto-generated method stub
		// 插入日志
		try {
			SysLogUtil sysLogsUtil = new SysLogUtil();
			sysLogsUtil.logSave("用户资料管理", 
					"更新用户信息："+userEntity.toString(),
					"更新");
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// 更新用户信息之后，基表安装对应的信息也相应修改：楼栋ID，楼栋名称、表身号、抄表标识码、住址
		DeviceInstallEntity dEntity = new DeviceInstallEntity();
		dEntity.setBuildingId(userEntity.getParentId());
		dEntity.setBuildingName(userEntity.getNodeName());
		dEntity.setDeviceNum(userEntity.getDeviceNum());
		dEntity.setCollectionNum(userEntity.getCollectionNum());
		dEntity.setHouseAddress(userEntity.getHouseAddress());
		dEntity.setHouseNum(userEntity.getHouseNum());
		dEntity.setLastValue(0.0);
		System.out.println("更新用户信息之后，基表安装对应的信息也相应修改：" + dEntity.toString());
		deviceInstallMapper.updateInstalls(dEntity);
		//更新完信息后，删除在wt_now表中的关联记录，更新wt_history中的数据 / 2019-12-13
		deviceInstallMapper.deleteWtNow(userEntity.getHouseNum());
		//判断抄表标识码是否有对应的户号
		int count=deviceInstallMapper.findHouseNoCount(dEntity.getDeviceNum());
		if(count>0) {
			deviceInstallMapper.updateHouseNo(dEntity.getHouseNum(), dEntity.getDeviceNum());
		}
		// 2019-12-13/END
		return userMapper.updateUser(userEntity);
	}

	@Override
	public int queryUsersCount(UserEntity userEntity, Integer page, Integer limit) {
		// TODO Auto-generated method stub
		return userMapper.queryUsersCount(userEntity, page, limit);
	}

	@Override
	public List<UserEntity> queryUserById(String title, int id) {
		// TODO Auto-generated method stub
		return userMapper.queryUserById(title, id);
	}

	@Override
	public List<UserEntity> queryUserByOthers(UserEntity userEntity, Integer page, Integer limit) {
		// TODO Auto-generated method stub
		return userMapper.queryUserByOthers(userEntity, page, limit);
	}

	@Override
	public int batchInsert(List<Object> excel2UserEntities) {
		// TODO Auto-generated method stub
		return userMapper.batchInsert(excel2UserEntities);
	}

}
